/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __NTHW_FPGA_RST_NT200A0X_H__
#define __NTHW_FPGA_RST_NT200A0X_H__

#include "nthw_drv.h"
#include "nthw_fpga_model.h"

struct NthwFpga_rst_nt200a0x
{
  int mnFpgaProductId;
  int mnFpgaVersion;
  int mnFpgaRevision;

  int mnHwId;
#if 0
  bool mbIsNT200A01;
#endif

  int mnSiLabsClockSynthModel;
  uint8_t mnSiLabsClockSynthI2cAddr;

  nthw_field_t* mpFldRstSys;
  nthw_field_t* mpFldRstSysMmcm;
  nthw_field_t* mpFldRstCoreMmcm;
  nthw_field_t* mpFldRstRpp;
  nthw_field_t* mpFldRstDdr4;
  nthw_field_t* mpFldRstSdc;
  nthw_field_t* mpFldRstPhy;
  nthw_field_t* mpFldRstSerdesRx;
  nthw_field_t* mpFldRstSerdesTx;
  nthw_field_t* mpFldRstSerdesRxDatapath;
  nthw_field_t* mpFldRstPcsRx;
  nthw_field_t* mpFldRstMacRx;
  nthw_field_t* mpFldRstMacTx;
  nthw_field_t* mpFldRstPtp;
  nthw_field_t* mpFldRstTs;
  nthw_field_t* mpFldRstPtpMmcm;
  nthw_field_t* mpFldRstTsMmcm;
  nthw_field_t* mpFldRstPeriph;
  nthw_field_t* mpFldRstTsmRefMmcm;
  nthw_field_t* mpFldRstTmc;

  /* CTRL register field pointers */
  nthw_field_t* mpFldCtrlTsClkSelOverride;
  nthw_field_t* mpFldCtrlTsClkSel;
  nthw_field_t* mpFldCtrlTsClkSelRef;
  nthw_field_t* mpFldCtrlPtpMmcmClkSel;

  /* STAT register field pointers */
  nthw_field_t* mpFldStatDdr4MmcmLocked;
  nthw_field_t* mpFldStatSysMmcmLocked;
  nthw_field_t* mpFldStatCoreMmcmLocked;
  nthw_field_t* mpFldStatDdr4PllLocked;
  nthw_field_t* mpFldStatPtpMmcmLocked;
  nthw_field_t* mpFldStatTsMmcmLocked;
  nthw_field_t* mpFldStatTsmRefMmcmLocked;

  /* STICKY register field pointers */
  nthw_field_t* mpFldStickyPtpMmcmUnlocked;
  nthw_field_t* mpFldStickyTsMmcmUnlocked;
  nthw_field_t* mpFldStickyDdr4MmcmUnlocked;
  nthw_field_t* mpFldStickyDdr4PllUnlocked;
  nthw_field_t* mpFldStickyCoreMmcmUnlocked;
  nthw_field_t* mpFldStickyPciSysMmcmUnlocked;
  nthw_field_t* mpFldStickyTsmRefMmcmUnlocked;

  /* POWER register field pointers */
  nthw_field_t* mpFldPowerPuPhy;
  nthw_field_t* mpFldPowerPuNseb;

  void (*resetSerdesRx)(struct NthwFpga_rst_nt200a0x *p, uint32_t intfNo, uint32_t rst);
  void (*pcs_rx_rst)(struct NthwFpga_rst_nt200a0x *p, uint32_t intfNo, uint32_t rst);
  void (*get_serdes_rx_rst)(struct NthwFpga_rst_nt200a0x *p, uint32_t intfNo, uint32_t *pSet);
  void (*get_pcs_rx_rst)(struct NthwFpga_rst_nt200a0x *p, uint32_t intfNo, uint32_t *pSet);
  bool (*is_rst_serdes_rx_datapath_implemented)(struct NthwFpga_rst_nt200a0x *p);
};

typedef struct NthwFpga_rst_nt200a0x NthwFpga_rst_nt200a0x_t;

int NthwFpga_rst_nt200a0x_init(struct FpgaInfo_s* pFpgaInfo, struct NthwFpga_rst_nt200a0x* pRst);
int NthwFpga_rst_nt200a0x_reset(nthw_fpga_t* pFpga, const struct NthwFpga_rst_nt200a0x* p);

#endif /* __NTHW_FPGA_RST_NT200A0X_H__ */
