/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_GPIO_PHY_H_
#define NTHW_GPIO_PHY_H_

#define GPIO_PHY_INTERFACES (2)

typedef struct {
  nthw_field_t*    CfgFldLpMode;  /* Cfg Low Power Mode */
  nthw_field_t*    CfgInt;        /* Cfg Port Interrupt */
  nthw_field_t*    CfgReset;      /* Cfg Reset */
  nthw_field_t*    CfgModPrs;     /* Cfg Module Present */
  nthw_field_t*    CfgPLLInt;     /* Cfg PLL Interrupt */
  nthw_field_t*    CfgPortRXLOS;  /* Emulate Cfg Port RXLOS */

  nthw_field_t*    GpioFldLpMode; /* Gpio Low Power Mode */
  nthw_field_t*    GpioInt;       /* Gpio Port Interrupt */
  nthw_field_t*    GpioReset;     /* Gpio Reset */
  nthw_field_t*    GpioModPrs;    /* Gpio Module Present */
  nthw_field_t*    GpioPLLInt;    /* Gpio PLL Interrupt */
  nthw_field_t*    GpioPortRXLOS; /* Emulate Gpio Port RXLOS */
} GpioPhyFields_t;

struct NthwGpioPhy
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModGpioPhy;
  int mnInstance;

  /* Registers */
  nthw_register_t* mpRegConfig;
  nthw_register_t* mpRegGpio;

  /* Fields */
  GpioPhyFields_t mpaFields[GPIO_PHY_INTERFACES];
};

typedef struct NthwGpioPhy NthwGpioPhy_t;
typedef struct NthwGpioPhy NthwGpioPhy;

NthwGpioPhy_t* NthwGpioPhy_new(void);
void NthwGpioPhy_delete(NthwGpioPhy_t* p);
int NthwGpioPhy_init(NthwGpioPhy_t* p, nthw_fpga_t* pFpga, int nInstance);

bool NthwGpioPhy_isLowPowerEnabled(NthwGpioPhy_t* p, uint8_t ifNo);
bool NthwGpioPhy_isInterruptSet(NthwGpioPhy_t* p, uint8_t ifNo);
bool NthwGpioPhy_isReset(NthwGpioPhy_t* p, uint8_t ifNo);
bool NthwGpioPhy_isModulePresent(NthwGpioPhy_t* p, uint8_t ifNo);
bool NthwGpioPhy_isPllInterruptSet(NthwGpioPhy_t* p, uint8_t ifNo);
bool NthwGpioPhy_isPortRXLOS(NthwGpioPhy_t* p, uint8_t ifNo);
void NthwGpioPhy_setLowPower(NthwGpioPhy_t* p, uint8_t ifNo, bool enable);
void NthwGpioPhy_setReset(NthwGpioPhy_t* p, uint8_t ifNo, bool enable);
void NthwGpioPhy_setPortRXLOS(NthwGpioPhy_t* p, uint8_t ifNo, bool enable);
void NthwGpioPhy_setCFGDefaultValues(NthwGpioPhy_t* p, uint8_t ifNo);

#endif /* NTHW_GPIO_PHY_H_ */
