/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_MAC10G_H_
#define NTHW_MAC10G_H_

struct NthwMac10g {
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModMac10g;
  int mnInstance;

  /* Control */
  nthw_field_t*    mpRxEnable;
  nthw_field_t*    mpRxPauseEnable;
  nthw_field_t*    mpTxEnable;
  nthw_field_t*    mpTxPauseEnable;
  nthw_field_t*    mpStrEnable;
  nthw_field_t*    mpRxMinFrame;

  /* MAC Address */
  nthw_field_t*    mpMacAddrLower;
  nthw_field_t*    mpMacAddrUpper;

  /* RX MAC Enable (RX_CONFIG) */
  nthw_field_t*    mpFldTsEop;
};


typedef struct NthwMac10g NthwMac10g_t;
typedef struct NthwMac10g NthwMac10g;

NthwMac10g_t* NthwMac10g_new(void);
void NthwMac10g_delete(NthwMac10g_t* p);
int NthwMac10g_init(NthwMac10g_t *p, nthw_fpga_t* pFpga, int nInstance);

void NthwMac10g_EnableStretch(NthwMac10g_t* p, bool enable);
void NthwMac10g_setRxEnable(NthwMac10g_t* p, bool enable);
void NthwMac10g_setTxEnable(NthwMac10g_t* p, bool enable);
void NthwMac10g_setRxPauseEnable(NthwMac10g_t* p, bool enable);
void NthwMac10g_setTxPauseEnable(NthwMac10g_t* p, bool enable);
void NthwMac10g_SetMacAddr(NthwMac10g_t* p, uint8_t macAddr[6]);
void NthwMac10g_setTsEop(NthwMac10g_t* p, uint32_t bitEop);
void NthwMac10g_setRxMinFrame(NthwMac10g_t* p, uint32_t min_rx_frame);

#endif /* NTHW_MAC10G_H_ */
