/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __NTHW_PCIE3_H__
#define __NTHW_PCIE3_H__

struct NthwPcie3
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModPcie3;
  int mnInstance;

  nthw_register_t* mpRegStatCtrl;
  nthw_field_t* mpFldStatCtrlReq;
  nthw_field_t* mpFldStatCtrlEna;

  nthw_register_t* mpRegStatRx;
  nthw_field_t* mpFldStatRxCounter;

  nthw_register_t* mpRegStatTx;
  nthw_field_t* mpFldStatTxCounter;

  nthw_register_t* mpRegStatRqRdy;
  nthw_field_t* mpFldStatRqRdyCounter;

  nthw_register_t* mpRegStatRqVld;
  nthw_field_t* mpFldStatRqVldCounter;

  nthw_register_t* mpRegStatus0;
  nthw_field_t* mpFldStatus0TagsInUse;

  nthw_register_t* mpRegStatRefClk;
  nthw_field_t* mpFldStatRefClkRefClk;

  nthw_register_t* mpRegRpToEpErr;
  nthw_field_t* mpFldRpToEpErrCor;
  nthw_field_t* mpFldRpToEpErrNonFatal;
  nthw_field_t* mpFldRpToEpErrFatal;

  nthw_register_t* mpRegEpToRpErr;
  nthw_field_t* mpFldEpToRpErrCor;
  nthw_field_t* mpFldEpToRpErrNonFatal;
  nthw_field_t* mpFldEpToRpErrFatal;

  nthw_register_t* mpRegSampleTime;
  nthw_field_t* mpFldSampleTime;

  nthw_register_t* mpRegPciEndPoint;
  nthw_field_t* mpFldPciEndPointIfId;
  nthw_field_t* mpFldPciEndPointSendMsg;
  nthw_field_t* mpFldPciEndPointGetMsg;
  nthw_field_t* mpFldPciEndPointDMAEP0AllowMask;
  nthw_field_t* mpFldPciEndPointDMAEP1AllowMask;

  nthw_register_t* mpRegPciE3MarkAdrLsb;
  nthw_field_t* mpFldPciE3MarkAdrLsbAdr;

  nthw_register_t* mpRegPciE3MarkAdrMsb;
  nthw_field_t* mpFldPciE3MarkAdrMsbAdr;

  nthw_register_t* mpRegPciTest0;
  nthw_field_t* mpFldPciTest0;

  nthw_register_t* mpRegPciTest1;
  nthw_field_t* mpFldPciTest1;

  nthw_register_t* mpRegPciTest2;
  nthw_field_t* mpFldPciTest2;

  nthw_register_t* mpRegPciTest3;
  nthw_field_t* mpFldPciTest3;
};

typedef struct NthwPcie3 NthwPcie3_t;
typedef struct NthwPcie3 NthwPcie3;

NthwPcie3_t* NthwPcie3_new(void);
void NthwPcie3_delete(NthwPcie3_t* p);
int NthwPcie3_init(NthwPcie3_t* p, nthw_fpga_t* pFpga, int nInstance);

int NthwPcie3_trigger_sample_time(NthwPcie3_t* p);

int NthwPcie3_StatReqEnable(NthwPcie3_t* p);
int NthwPcie3_StatReqDisable(NthwPcie3_t* p);
#if 0
int NthwPcie3_StatSample(NthwPcie3_t* p, uint64_t* pRxRate, uint64_t* pTxRate, uint64_t* pRefClkCnt, uint64_t* pTagsInUse, uint64_t* pRdErrCnt, uint64_t* pWrErrCnt);
#endif

int NthwPcie3_getStat(NthwPcie3_t* p, uint32_t* pRxCnt, uint32_t* pTxCnt, uint32_t* pRefClkCnt, uint32_t* pTgUnitSize, uint32_t* pTgRefFreq, uint32_t* pTagUseCnt, uint32_t* pRqRdyCnt, uint32_t* pRqVldCnt);
int NthwPcie3_getStatRate(NthwPcie3_t* p, uint64_t* pPciRxRate, uint64_t* pPciTxRate, uint64_t* pRefClkCnt, uint64_t* pTagUseCnt, uint64_t* pPciNtBusUtil, uint64_t* pPciXilBusUtil);

int NthwPcie3_EndPointCountersSamplePre(NthwPcie3_t* p, struct NthwHif_EndPointCounters* epc);
int NthwPcie3_EndPointCountersSamplePost(NthwPcie3_t* p, struct NthwHif_EndPointCounters* epc);

#endif /* __NTHW_PCIE3_H__ */
