/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __NTHW_PHY_TILE_H__
#define __NTHW_PHY_TILE_H__

#include "nthw_fpga_model.h"

struct nt_phy_tile
{
  nthw_fpga_t* mpFpga;

  nthw_module_t* mModPhyTile;

  int mnPhyTileInstance;

  nthw_register_t* mpRegPortXcvrBase[2][4];
  nthw_field_t*    mpFldPortXcvrBasePtr[2][4];
  nthw_field_t*    mpFldPortXcvrBaseBusy[2][4];
  nthw_field_t*    mpFldPortXcvrBaseCmd[2][4];

  nthw_register_t* mpRegPortXcvrData[2][4];
  nthw_field_t*    mpFldPortXcvrDataData[2][4];

  nthw_register_t* mpRegPortEthBase[2][4];
  nthw_field_t*    mpFldPortEthBasePtr[2][4];
  nthw_field_t*    mpFldPortEthBaseBusy[2][4];
  nthw_field_t*    mpFldPortEthBaseCmd[2][4];

  nthw_register_t* mpRegPortEthData[2][4];
  nthw_field_t*    mpFldPortEthDataData[2][4];

  nthw_register_t* mpRegLinkSummary[2];
  nthw_field_t*    mpFldLinkSummaryNtPhyLinkState[2];
  nthw_field_t*    mpFldLinkSummaryLlNtPhyLinkState[2];
  nthw_field_t*    mpFldLinkSummaryLinkDownCnt[2];
  nthw_field_t*    mpFldLinkSummaryLlRxBlockLock[2];
  nthw_field_t*    mpFldLinkSummaryLlRxAmLock[2];
  nthw_field_t*    mpFldLinkSummaryLhRxHighBitErrorRate[2];
  nthw_field_t*    mpFldLinkSummaryLhReceivedLocalFault[2];
  nthw_field_t*    mpFldLinkSummaryLhRemoteFault[2];

  nthw_register_t* mpRegPortStatus;
  nthw_field_t*    mpFldPortStatusRxPcsFullyAligned[2];
  nthw_field_t*    mpFldPortStatusRxHiBer[2];
  nthw_field_t*    mpFldPortStatusRxRemoteFault[2];
  nthw_field_t*    mpFldPortStatusRxLocalFault[2];
  nthw_field_t*    mpFldPortStatusRxAmLock[2];

  nthw_register_t* mpRegPortConfig;
  nthw_field_t*    mpFldPortConfigDynReset;
  nthw_field_t*    mpFldPortConfigReset[2];
  nthw_field_t*    mpFldPortConfigRxReset[2];
  nthw_field_t*    mpFldPortConfigTxReset[2];

  nthw_register_t* mpRegPortComp[2];
  nthw_field_t*    mpFldPortCompRxCompensation[2];
  nthw_field_t*    mpFldPortCompTxCompensation[2];

  nthw_register_t* mpRegDynReconfigBase;
  nthw_field_t*    mpFldDynReconfigBasePtr;
  nthw_field_t*    mpFldDynReconfigBaseBusy;
  nthw_field_t*    mpFldDynReconfigBaseCmd;

  nthw_register_t* mpRegDynReconfigData;
  nthw_field_t*    mpFldDynReconfigDataData;

  nthw_register_t* mpRegScratch;
  nthw_field_t*    mpFldScratchData;

  nthw_register_t* mpRegDrCfgStatus;
  nthw_field_t*    mpFldDrCfgStatusCurrProfileId;
  nthw_field_t*    mpFldDrCfgStatusInProgress;
  nthw_field_t*    mpFldDrCfgStatusError;
};

typedef struct nt_phy_tile nthw_phy_tile_t;
typedef struct nt_phy_tile nt_phy_tile;

nthw_phy_tile_t* nthw_phy_tile_new(void);
void nthw_phy_tile_delete(nthw_phy_tile_t* p);
int nthw_phy_tile_init(nthw_phy_tile_t *p, nthw_fpga_t *pFpga, int mnPhyTileInstance);
uint32_t nthw_phy_tile_getTxPolInv(nthw_phy_tile_t *p, uint8_t intfNo, uint8_t lane);
uint32_t nthw_phy_tile_getRxPolInv(nthw_phy_tile_t *p, uint8_t intfNo, uint8_t lane);
void nthw_phy_tile_setTxPolInv(nthw_phy_tile_t *p, uint8_t intfNo, uint8_t lane, bool invert);
void nthw_phy_tile_setRxPolInv(nthw_phy_tile_t *p, uint8_t intfNo, uint8_t lane, bool invert);
void nthw_phy_tile_setHostLoopback(nthw_phy_tile_t *p, uint8_t intfNo, uint8_t lane, bool enable);
uint32_t nthw_phy_tile_getHostLoopback(nthw_phy_tile_t *p, uint8_t intfNo, uint8_t lane);
void nthw_phy_tile_setTxEqualization(nthw_phy_tile_t *p, uint8_t intfNo, uint8_t lane, uint32_t preTap2, uint32_t mainTap, uint32_t preTap1, uint32_t postTap1);
void nthw_phy_tile_getTxEqualization(nthw_phy_tile_t *p, uint8_t intfNo, uint8_t lane);
void nthw_phy_tile_getLinkSummary(nthw_phy_tile_t *p, uint32_t* pNtPhyLinkState,
                                   uint32_t* pLlNtPhyLinkState,
                                   uint32_t* pLhLocalFault,
                                   uint32_t* pLhRemoteFault,
                                   uint8_t   index);
void nthw_phy_tile_setTxReset(nthw_phy_tile_t *p, uint8_t intfNo, bool reset);
void nthw_phy_tile_setRxReset(nthw_phy_tile_t *p, uint8_t intfNo, bool reset);
bool nthw_phy_tile_readFecEnabledByScratch(nthw_phy_tile_t *p, uint8_t intfNo);
void nthw_phy_tile_writeFecEnabledByScratch(nthw_phy_tile_t *p, uint8_t intfNo, bool FecEnabled);
void nthw_phy_tile_setDynReset(nthw_phy_tile_t *p, uint8_t reset);
bool nthw_phy_tile_getRxHiBer(nthw_phy_tile_t *p, uint8_t intfNo);
bool nthw_phy_tile_getRxAmLock(nthw_phy_tile_t *p, uint8_t intfNo);
void nthw_phy_tile_setTimestampCompRx(nthw_phy_tile_t *p, uint8_t intfNo, uint32_t value);
uint32_t nthw_phy_tile_getTimestampCompRx(nthw_phy_tile_t *p, uint8_t intfNo);
void nthw_phy_tile_setTimestampCompTx(nthw_phy_tile_t *p, uint8_t intfNo, uint32_t value);
void nthw_phy_tile_write_dr_ctrl_mc_cfgcsr_reg(nthw_phy_tile_t *p, uint32_t offset, uint32_t wdata);
uint32_t nthw_phy_tile_polling_for_bit_dr_ctrl(nthw_phy_tile_t *p, uint32_t channel, uint32_t offset, uint32_t bit_offset);
uint32_t nthw_phy_tile_readEth(nthw_phy_tile_t *p, uint8_t intfNo, uint8_t lane, uint32_t address);
void nthw_phy_tile_writeEth(nthw_phy_tile_t *p, uint8_t intfNo, uint8_t lane, uint32_t address, uint32_t data);
bool nthw_phy_tile_configureFec(nthw_phy_tile_t *p, uint8_t intfNo, bool enable);

#endif /* __NTHW_PHY_TILE_H__ */
