/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_QM_HPP_
#define NTHW_QM_HPP_

enum NthwQm_QueuePriority {
    NTHW_QM_QUEUE_PRIORITY_MAX=0x00,
    NTHW_QM_QUEUE_PRIORITY_HIGH=0x01,
    NTHW_QM_QUEUE_PRIORITY_LOW=0x02,
    NTHW_QM_QUEUE_PRIORITY_MIN=0x03,
};

struct NthwQm
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModQm;
  int mnInstance;

  uint32_t mnQueues;
  uint32_t mnBlocks;
  uint32_t mnCells;
  uint32_t mnCellSize;

  /* 1: QM_CTRL */
  nthw_register_t* mpRegCtrl;
  nthw_field_t*    mpFldCtrlEnable;
  /*
   * 0: Priorities are controlled by the priorities set in the Priority_Levels table.
   * 1: Each Queues priority level depends on the fill level
   *    (of Queue or Group Queue belongs to) for that Queue.
   * 2: Each Queues priority level depends on the how long time since the Cells were written
   *    to SDRAM.
   */
  nthw_field_t*    mpFldCtrlPriorityScheme; /* 1: this is currently the only scheme supported */
  nthw_field_t*    mpFldCtrlActiveQueues;

  /* 2: QM_DEBUG_CRC */
  nthw_register_t* mpRegDebugCrc;
  nthw_field_t*    mpFldDebugCrcForceError;

  /* 3: QM_DEBUG_SDRAM_SIZE: default 4 MCells each 2 KByte: 8 GByte SDRAM */
  nthw_register_t* mpRegDebugSdramSize;
  nthw_field_t*    mpFldDebugSdramSizeMask;

  /* 3: QM_DEBUG_BLOCK_CELLS: default 4 MCells each 2 KByte: 8 GByte SDRAM */
  nthw_register_t* mpRegDebugBlockSize;
  nthw_field_t*    mpFldDebugBlockSizeCells;

  /* 4: QM_STATUS_SDRAM_BLOCK_USAGE */
  nthw_register_t* mpRegStatusSdramBlockUsage;

  nthw_field_t*    mpFldStatusSdramBlockUsageCount; /*
                                                     * SDRAM Block Usage, shows the current number of Blocks used in SDRAM. MAX = 2048 Blocks
                                                     * 1 Block = 2048 Cells; 1 Cell = 2 KByte; 1 Block = 2048 * 2048 = 4 MByte
                                                     */

  /* 5: QM_STATUS_SDRAM_BLOCK_MAX_USAGE */
  nthw_register_t* mpRegStatusSdramBlockMaxUsage;
  /* SDRAM Block Max Usage, shows the maximum number of Blocks used in SDRAM. */
  nthw_field_t*    mpFldStatusSdramBlockMaxUsageCount;

  /* 6: QM_STATUS_SDRAM_CELL_USAGE */
  nthw_register_t* mpRegStatusSdramCellUsage;
  /* SDRAM Cell Usage, shows the number of Cells used in SDRAM. */
  nthw_field_t*    mpFldStatusSdramCellUsageCount;

  /* 7: QM_STATUS_SDRAM_CELL_MAX_USAGE */
  nthw_register_t* mpRegStatusSdramCellMaxUsage;
  /* SDRAM Cell Usage, shows the maximum number of Cells used in SDRAM. */
  nthw_field_t*    mpFldStatusSdramCellMaxUsageCount;

  /* 8: QM_STATUS_CRC_ERROR */
  nthw_register_t* mpRegStatusCrcError;
  /* SDRAM Cells with CRC error */
  nthw_field_t*    mpFldStatusCrcErrorCount;

  /* 9: QM_STATUS_FLUSH_DROP */
  nthw_register_t* mpRegStatusFlushDrop;
  /* SDRAM Cells which are dropped due to Flushes */
  nthw_field_t*    mpFldStatusFlushDropCount;

  /* 10: QM_STATUS_BLOCKED */
  nthw_register_t* mpRegStatusBlocked;
  /*
   * One bit for each Queue indicates whether that queue blocked by next module in RX pipe
   * (QHM).
   */
  nthw_field_t*    mpFldStatusBlockedBitStatus;

  /* 11: QM_STATUS_BLOCKING */
  nthw_register_t* mpRegStatusBlocking;
  /*
   * One bit for each Queue indicates whether that queue is blocking previous module in RX
   * pipe (CB).
   */
  nthw_field_t*    mpFldStatusBlockingBitStatus;

  /* 12: QM_STATUS_EMPTY */
  nthw_register_t* mpRegStatusEmpty;
  /* One bit for each Queue indicates whether that queue is empty. */
  nthw_field_t*    mpFldStatusEmptyBitStatus;

  /*
   * ==>> 0: Priorities are controlled by the priorities set in the Priority_Levels table.
   * 13: QM_PRIO_LEVELS_MEM_CTRL
   */
  nthw_register_t* mpRegPrioLevelsMemCtrl;
  nthw_field_t*    mpFldPrioLevelsMemCtrlAdr;
  nthw_field_t*    mpFldPrioLevelsMemCtrlCnt;

  /* 14: QM_PRIO_LEVELS_MEM_DATA */
  nthw_register_t* mpRegPrioLevelsMemData;
  nthw_field_t*    mpFldPrioLevelsMemDataPriority;

  /*
   * ==>>1: Group mapping
   * 15: QM_GROUP_MAPPING_MEM_CTRL
   */
  nthw_register_t* mpRegGroupMappingMemCtrl;
  nthw_field_t*    mpFldGroupMappingMemCtrlAdr;
  nthw_field_t*    mpFldGroupMappingMemCtrlCnt;

  /* 16: QM_GROUP_MAPPING_MEM_DATA */
  nthw_register_t* mpRegGroupMappingMemData;
  nthw_field_t*    mpFldGroupMappingMemDataGroup0;
  nthw_field_t*    mpFldGroupMappingMemDataGroup1;
  nthw_field_t*    mpFldGroupMappingMemDataGroup2;

  /*
   * ==>> 2: Queue limit
   * 17: QM_QUEUE_LIMIT_MEM_CTRL
   */
  nthw_register_t* mpRegQueueLimitMemCtrl;
  nthw_field_t*    mpFldQueueLimitMemCtrlAdr;
  nthw_field_t*    mpFldQueueLimitMemCtrlCnt;

  /* 18: QM_QUEUE_LIMIT_MEM_DATA */
  nthw_register_t* mpRegQueueLimitMemData;
  nthw_field_t*    mpFldQueueLimitMemDataLimit;

  /*
   * ==>>3: Group limit
   * 19: QM_GROUP_LIMIT_MEM_CTRL
   */
  nthw_register_t* mpRegGroupLimitMemCtrl;
  nthw_field_t*    mpFldGroupLimitMemCtrlAdr;
  nthw_field_t*    mpFldGroupLimitMemCtrlCnt;

  /* 20: QM_GROUP_LIMIT_MEM_DATA */
  nthw_register_t* mpRegGroupLimitMemData;
  nthw_field_t*    mpFldGroupLimitMemDataLimit;


};

typedef struct NthwQm NthwQm_t;
typedef struct NthwQm NthwQm;

NthwQm_t* NthwQm_new(void);
void NthwQm_delete(NthwQm_t* p);
int NthwQm_init(NthwQm_t* p, nthw_fpga_t* pFpga, int nInstance);

void NthwQm_enable(NthwQm_t* p);
void NthwQm_disable(NthwQm_t* p);
uint32_t NthwQm_getEnable(NthwQm_t* p);
void NthwQm_setEnable(NthwQm_t* p, uint32_t value);
void NthwQm_setActiveQueues(NthwQm_t* p, int value);

uint32_t NthwQm_getStatusCrcErrorCount(NthwQm_t* p);
void NthwQm_setStatusCrcErrorCount(NthwQm_t* p, uint32_t value);

uint32_t NthwQm_getStatusSdramBlockUsageCount(NthwQm_t* p);
uint32_t NthwQm_getStatusSdramBlockMaxUsageCount(NthwQm_t* p);
uint32_t NthwQm_getStatusSdramCellUsageCount(NthwQm_t* p);
uint32_t NthwQm_getStatusSdramCellMaxUsageCount(NthwQm_t* p);
uint32_t NthwQm_getDebugSdramSizeMask(NthwQm_t* p);
uint32_t NthwQm_getDebugBlockSizeCells(NthwQm_t* p);

void NthwQm_setQueuePriority(NthwQm_t* p, uint16_t queue, uint8_t priority);
void NthwQm_setGroupLimit(NthwQm_t* p, uint16_t group, uint16_t limit);
void NthwQm_setQueueLimit(NthwQm_t* p, uint16_t queue, uint16_t limit);
void NthwQm_setQueueGroupMapping(NthwQm_t* p, uint16_t queue, uint16_t group0, uint16_t group1, uint16_t group2 );
void NthwQm_setPriorityScheme(NthwQm_t* p, uint8_t priority_scheme);
void NthwQm_setConfiguration(NthwQm_t* p, uint8_t variant);

uint32_t NthwQm_getParamQueues(NthwQm_t* p);
uint32_t NthwQm_getParamSdramCells(NthwQm_t* p);
uint32_t NthwQm_getParamSdramCellSize(NthwQm_t* p);
uint32_t NthwQm_getParamSdramBlocks(NthwQm_t* p);

/* set QM config variant 1 */
void NthwQm_setConfigurationVariant1(NthwQm_t* p);

#endif /* NTHW_QM_H_ */
