/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_RMC_H_
#define NTHW_RMC_H_

struct NthwRmc
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModRmc;
  int mnInstance;

  int mnPorts;
  int mnNims;
  bool mbIsVswitch;

  bool mbAdministrativeBlock;

  /* RMC CTRL register */
  nthw_register_t* mpRegCtrl;
  nthw_field_t* mpFldCtrlBlockStatDrop;
  nthw_field_t* mpFldCtrlBlockKeepAlive;
  nthw_field_t* mpFldCtrlBlockMacPort;

  /* RMC Status register */
  nthw_register_t* mpRegStatus;
  nthw_field_t* mpFldSfRamOf;
  nthw_field_t* mpFldDescrFifoOf;

  /* RMC DBG register */
  nthw_register_t* mpRegDbg;
  nthw_field_t* mpFldDbgMerge;

  /* RMC MAC_IF register */
  nthw_register_t* mpRegMacIf;
  nthw_field_t* mpFldMacIfErr;
};

typedef struct NthwRmc NthwRmc_t;
typedef struct NthwRmc NthwRmc;

NthwRmc_t* NthwRmc_new(void);
void NthwRmc_delete(NthwRmc_t* p);
int NthwRmc_init(NthwRmc_t *p, nthw_fpga_t* pFpga, int nInstance);

uint32_t NthwRmc_getMacBlock(NthwRmc_t* p);
void NthwRmc_setMacBlock(NthwRmc_t* p, uint32_t mask);
void NthwRmc_block(NthwRmc_t* p);
void NthwRmc_unblock(NthwRmc_t* p, bool bIsSlave);
void NthwRmc_administrativeBlock(NthwRmc_t* p);

uint32_t NthwRmc_getStatusSfRamOf(NthwRmc_t* p);
uint32_t NthwRmc_getStatusDescrFifoOf(NthwRmc_t* p);
uint32_t NthwRmc_getDbgMerge(NthwRmc_t* p);
uint32_t NthwRmc_getMacIfErr(NthwRmc_t* p);

#endif /* NTHW_RMC_H_ */
