/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_CAT_H__
#define __FLOW_NTHW_CAT_H__

#include <stdint.h> /* uint32_t */
#include "nthw_fpga_model.h"

struct CatNthw;

typedef struct CatNthw CatNthw_t;
#if 0
typedef struct CatNthw CatNthw;
#endif

struct CatNthw *CatNthw_new(void);
void CatNthw_delete(struct CatNthw *p);
int CatNthw_init(struct CatNthw *p, nthw_fpga_t *pFpga, int nInstance);

int CatNthw_setup(struct CatNthw *p, int nIdx, int nIdxCnt);
void catNthw_setDebugMode(struct CatNthw *p, unsigned int nDebugMode);

#if 0
  uint32_t GetVersionMajor(void) {return (mCat.getVersion()).getMajorVersion();}
  uint32_t GetVersionMinor(void) {return (mCat.getVersion()).getMinorVersion();}
  void SetDebugOutput(int on) {if (on) mCat.setDebugMode(Register_t::ON_WRITE); else mCat.setDebugMode(Register_t::NO_DEBUG);}
#endif

/* CFN */
void CatNthw_CfnSelect(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnCnt(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnEnable(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnInv(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcInv(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcIsl(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcCfp(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcMac(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcL2(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcVnTag(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcVlan(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcMpls(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcL3(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcFrag(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcIpProt(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcL4(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcTunnel(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcTnlL2(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcTnlVlan(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcTnlMpls(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcTnlL3(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcTnlFrag(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcTnlIpProt(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPtcTnlL4(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnErrInv(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnErrCv(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnErrFcs(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnErrTrunc(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnErrL3Cs(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnErrL4Cs(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnErrTnlL3Cs(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnErrTnlL4Cs(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnErrTtlExp(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnErrTnlTtlExp(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnMacPort(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPmCmp(const struct CatNthw *p, const uint32_t *val);
void CatNthw_CfnPmDct(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPmExtInv(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPmCmb(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPmAndInv(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPmOrInv(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnPmInv(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnLc(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnLcInv(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnKm0Or(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnKm1Or(const struct CatNthw *p, uint32_t val);
void CatNthw_CfnFlush(const struct CatNthw *p);
/* KCE 0/1 */
void CatNthw_KceSelect(const struct CatNthw *p, int index, uint32_t val);
void CatNthw_KceCnt(const struct CatNthw *p, int index, uint32_t val);
void CatNthw_KceEnable(const struct CatNthw *p, int index, uint32_t val);
void CatNthw_KceFlush(const struct CatNthw *p, int index);
/* KCS 0/1 */
void CatNthw_KcsSelect(const struct CatNthw *p, int index, uint32_t val);
void CatNthw_KcsCnt(const struct CatNthw *p, int index, uint32_t val);
void CatNthw_KcsCategory(const struct CatNthw *p, int index, uint32_t val);
void CatNthw_KcsFlush(const struct CatNthw *p, int index);
/* FTE 0/1 */
void CatNthw_FteSelect(const struct CatNthw *p, int index, uint32_t val);
void CatNthw_FteCnt(const struct CatNthw *p, int index, uint32_t val);
void CatNthw_FteEnable(const struct CatNthw *p, int index, uint32_t val);
void CatNthw_FteFlush(const struct CatNthw *p, int index);
/* CTE */
void CatNthw_CteSelect(const struct CatNthw *p, uint32_t val);
void CatNthw_CteCnt(const struct CatNthw *p, uint32_t val);
void CatNthw_CteEnableCol(const struct CatNthw *p, uint32_t val);
void CatNthw_CteEnableCor(const struct CatNthw *p, uint32_t val);
void CatNthw_CteEnableHsh(const struct CatNthw *p, uint32_t val);
void CatNthw_CteEnableQsl(const struct CatNthw *p, uint32_t val);
void CatNthw_CteEnableIpf(const struct CatNthw *p, uint32_t val);
void CatNthw_CteEnableSlc(const struct CatNthw *p, uint32_t val);
void CatNthw_CteEnablePdb(const struct CatNthw *p, uint32_t val);
void CatNthw_CteEnableMsk(const struct CatNthw *p, uint32_t val);
void CatNthw_CteEnableHst(const struct CatNthw *p, uint32_t val);
void CatNthw_CteEnableEpp(const struct CatNthw *p, uint32_t val);
void CatNthw_CteEnableTpe(const struct CatNthw *p, uint32_t val);
void CatNthw_CteEnableRrb(const struct CatNthw *p, uint32_t val);
void CatNthw_CteFlush(const struct CatNthw *p);
/* CTS */
void CatNthw_CtsSelect(const struct CatNthw *p, uint32_t val);
void CatNthw_CtsCnt(const struct CatNthw *p, uint32_t val);
void CatNthw_CtsFlush(const struct CatNthw *p);
void CatNthw_CtsCatA(const struct CatNthw *p, uint32_t val);
void CatNthw_CtsCatB(const struct CatNthw *p, uint32_t val);
/* COT */
void CatNthw_CotSelect(const struct CatNthw *p, uint32_t val);
void CatNthw_CotCnt(const struct CatNthw *p, uint32_t val);
void CatNthw_CotColor(const struct CatNthw *p, uint32_t val);
void CatNthw_CotKm(const struct CatNthw *p, uint32_t val);
void CatNthw_CotNfvSb(const struct CatNthw *p, uint32_t val);
void CatNthw_CotFlush(const struct CatNthw *p);
/* CCT */
void CatNthw_CctSelect(const struct CatNthw *p, uint32_t val);
void CatNthw_CctCnt(const struct CatNthw *p, uint32_t val);
void CatNthw_CctColor(const struct CatNthw *p, uint32_t val);
void CatNthw_CctKm(const struct CatNthw *p, uint32_t val);
void CatNthw_CctFlush(const struct CatNthw *p);
/* EXO */
void CatNthw_ExoSelect(const struct CatNthw *p, uint32_t val);
void CatNthw_ExoCnt(const struct CatNthw *p, uint32_t val);
void CatNthw_ExoDyn(const struct CatNthw *p, uint32_t val);
void CatNthw_ExoOfs(const struct CatNthw *p, int32_t val);
void CatNthw_ExoFlush(const struct CatNthw *p);
/* RCK */
void CatNthw_RckSelect(const struct CatNthw *p, uint32_t val);
void CatNthw_RckCnt(const struct CatNthw *p, uint32_t val);
void CatNthw_RckData(const struct CatNthw *p, uint32_t val);
void CatNthw_RckFlush(const struct CatNthw *p);
/* LEN */
void CatNthw_LenSelect(const struct CatNthw *p, uint32_t val);
void CatNthw_LenCnt(const struct CatNthw *p, uint32_t val);
void CatNthw_LenLower(const struct CatNthw *p, uint32_t val);
void CatNthw_LenUpper(const struct CatNthw *p, uint32_t val);
void CatNthw_LenDyn1(const struct CatNthw *p, uint32_t val);
void CatNthw_LenDyn2(const struct CatNthw *p, uint32_t val);
void CatNthw_LenInv(const struct CatNthw *p, uint32_t val);
void CatNthw_LenFlush(const struct CatNthw *p);
/* KCC */
void CatNthw_KccSelect(const struct CatNthw *p, uint32_t val);
void CatNthw_KccCnt(const struct CatNthw *p, uint32_t val);
void CatNthw_KccKey(const struct CatNthw *p, uint32_t *val);
void CatNthw_KccCategory(const struct CatNthw *p, uint32_t val);
void CatNthw_KccId(const struct CatNthw *p, uint32_t val);
void CatNthw_KccFlush(const struct CatNthw *p);
/* CCE */
void CatNthw_CceSelect(const struct CatNthw *p, uint32_t val);
void CatNthw_CceCnt(const struct CatNthw *p, uint32_t val);
void CatNthw_CceDataImm(const struct CatNthw *p, uint32_t val);
void CatNthw_CceDataInd(const struct CatNthw *p, uint32_t val);
void CatNthw_CceFlush(const struct CatNthw *p);
/* CCS */
void CatNthw_CcsSelect(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsCnt(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataCorEn(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataCor(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataHshEn(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataHsh(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataQslEn(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataQsl(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataIpfEn(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataIpf(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataSlcEn(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataSlc(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataPdbEn(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataPdb(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataMskEn(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataMsk(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataHstEn(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataHst(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataEppEn(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataEpp(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataTpeEn(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataTpe(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataRrbEn(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataRrb(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataSb0Type(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataSb0Data(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataSb1Type(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataSb1Data(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataSb2Type(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsDataSb2Data(const struct CatNthw *p, uint32_t val);
void CatNthw_CcsFlush(const struct CatNthw *p);


struct CatNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;
  nthw_module_t *mCat;
  int mKmIfCnt;

  nthw_register_t *mpCfnCtrl;
  nthw_field_t *mpCfnAddr;
  nthw_field_t *mpCfnCnt;
  nthw_register_t *mpCfnData;
  nthw_field_t *mpCfnDataEnable;
  nthw_field_t *mpCfnDataInv;
  nthw_field_t *mpCfnDataPtcInv;
  nthw_field_t *mpCfnDataPtcIsl;
  nthw_field_t *mpCfnDataPtcCfp;
  nthw_field_t *mpCfnDataPtcMac;
  nthw_field_t *mpCfnDataPtcL2;
  nthw_field_t *mpCfnDataPtcVnTag;
  nthw_field_t *mpCfnDataPtcVlan;
  nthw_field_t *mpCfnDataPtcMpls;
  nthw_field_t *mpCfnDataPtcL3;
  nthw_field_t *mpCfnDataPtcFrag;
  nthw_field_t *mpCfnDataPtcIpProt;
  nthw_field_t *mpCfnDataPtcL4;
  nthw_field_t *mpCfnDataPtcTunnel;
  nthw_field_t *mpCfnDataPtcTnlL2;
  nthw_field_t *mpCfnDataPtcTnlVlan;
  nthw_field_t *mpCfnDataPtcTnlMpls;
  nthw_field_t *mpCfnDataPtcTnlL3;
  nthw_field_t *mpCfnDataPtcTnlFrag;
  nthw_field_t *mpCfnDataPtcTnlIpProt;
  nthw_field_t *mpCfnDataPtcTnlL4;
  nthw_field_t *mpCfnDataErrInv;
  nthw_field_t *mpCfnDataErrCv;
  nthw_field_t *mpCfnDataErrFcs;
  nthw_field_t *mpCfnDataErrTrunc;
  nthw_field_t *mpCfnDataErrL3Cs;
  nthw_field_t *mpCfnDataErrL4Cs;
  nthw_field_t *mpCfnDataErrTnlL3Cs;
  nthw_field_t *mpCfnDataErrTnlL4Cs;
  nthw_field_t *mpCfnDataErrTtlExp;
  nthw_field_t *mpCfnDataErrTnlTtlExp;
  nthw_field_t *mpCfnDataMacPort;
  nthw_field_t *mpCfnDataPmCmp;
  nthw_field_t *mpCfnDataPmDct;
  nthw_field_t *mpCfnDataPmExtInv;
  nthw_field_t *mpCfnDataPmCmb;
  nthw_field_t *mpCfnDataPmAndInv;
  nthw_field_t *mpCfnDataPmOrInv;
  nthw_field_t *mpCfnDataPmInv;
  nthw_field_t *mpCfnDataLc;
  nthw_field_t *mpCfnDataLcInv;
  nthw_field_t *mpCfnDataKm0Or;
  nthw_field_t *mpCfnDataKm1Or;

  nthw_register_t *mpKceCtrl[2];
  nthw_field_t *mpKceAddr[2];
  nthw_field_t *mpKceCnt[2];
  nthw_register_t *mpKceData[2];
  nthw_field_t *mpKceDataEnable[2];

  nthw_register_t *mpKcsCtrl[2];
  nthw_field_t *mpKcsAddr[2];
  nthw_field_t *mpKcsCnt[2];
  nthw_register_t *mpKcsData[2];
  nthw_field_t *mpKcsDataCategory[2];

  nthw_register_t *mpFteCtrl[2];
  nthw_field_t *mpFteAddr[2];
  nthw_field_t *mpFteCnt[2];
  nthw_register_t *mpFteData[2];
  nthw_field_t *mpFteDataEnable[2];

  nthw_register_t *mpCteCtrl;
  nthw_field_t *mpCteAddr;
  nthw_field_t *mpCteCnt;
  nthw_register_t *mpCteData;
  nthw_field_t *mpCteDataCol;
  nthw_field_t *mpCteDataCor;
  nthw_field_t *mpCteDataHsh;
  nthw_field_t *mpCteDataQsl;
  nthw_field_t *mpCteDataIpf;
  nthw_field_t *mpCteDataSlc;
  nthw_field_t *mpCteDataPdb;
  nthw_field_t *mpCteDataMsk;
  nthw_field_t *mpCteDataHst;
  nthw_field_t *mpCteDataEpp;
  nthw_field_t *mpCteDataTpe;
  nthw_field_t *mpCteDataRrb;

  nthw_register_t *mpCtsCtrl;
  nthw_field_t *mpCtsAddr;
  nthw_field_t *mpCtsCnt;
  nthw_register_t *mpCtsData;
  nthw_field_t *mpCtsDataCatA;
  nthw_field_t *mpCtsDataCatB;

  nthw_register_t *mpCotCtrl;
  nthw_field_t *mpCotAddr;
  nthw_field_t *mpCotCnt;
  nthw_register_t *mpCotData;
  nthw_field_t *mpCotDataColor;
  nthw_field_t *mpCotDataKm;
  nthw_field_t *mpCotDataNfvSb;

  nthw_register_t *mpCctCtrl;
  nthw_field_t *mpCctAddr;
  nthw_field_t *mpCctCnt;
  nthw_register_t *mpCctData;
  nthw_field_t *mpCctDataColor;
  nthw_field_t *mpCctDataKm;

  nthw_register_t *mpExoCtrl;
  nthw_field_t *mpExoAddr;
  nthw_field_t *mpExoCnt;
  nthw_register_t *mpExoData;
  nthw_field_t *mpExoDataDyn;
  nthw_field_t *mpExoDataOfs;

  nthw_register_t *mpRckCtrl;
  nthw_field_t *mpRckAddr;
  nthw_field_t *mpRckCnt;
  nthw_register_t *mpRckData;

  nthw_register_t *mpLenCtrl;
  nthw_field_t *mpLenAddr;
  nthw_field_t *mpLenCnt;
  nthw_register_t *mpLenData;
  nthw_field_t *mpLenDataLower;
  nthw_field_t *mpLenDataUpper;
  nthw_field_t *mpLenDataDyn1;
  nthw_field_t *mpLenDataDyn2;
  nthw_field_t *mpLenDataInv;
  nthw_register_t *mpKccCtrl;
  nthw_field_t *mpKccAddr;
  nthw_field_t *mpKccCnt;

  nthw_register_t *mpKccData;
  nthw_field_t *mpKccDataKey;
  nthw_field_t *mpKccDataCategory;
  nthw_field_t *mpKccDataId;

  nthw_register_t *mpCceCtrl;
  nthw_field_t *mpCceAddr;
  nthw_field_t *mpCceCnt;

  nthw_register_t *mpCceData;
  nthw_field_t *mpCceDataImm;
  nthw_field_t *mpCceDataInd;

  nthw_register_t *mpCcsCtrl;
  nthw_field_t *mpCcsAddr;
  nthw_field_t *mpCcsCnt;

  nthw_register_t *mpCcsData;
  nthw_field_t *mpCcsDataCorEn;
  nthw_field_t *mpCcsDataCor;

  nthw_field_t *mpCcsDataHshEn;
  nthw_field_t *mpCcsDataHsh;
  nthw_field_t *mpCcsDataQslEn;
  nthw_field_t *mpCcsDataQsl;
  nthw_field_t *mpCcsDataIpfEn;
  nthw_field_t *mpCcsDataIpf;
  nthw_field_t *mpCcsDataSlcEn;
  nthw_field_t *mpCcsDataSlc;
  nthw_field_t *mpCcsDataPdbEn;
  nthw_field_t *mpCcsDataPdb;
  nthw_field_t *mpCcsDataMskEn;
  nthw_field_t *mpCcsDataMsk;
  nthw_field_t *mpCcsDataHstEn;
  nthw_field_t *mpCcsDataHst;
  nthw_field_t *mpCcsDataEppEn;
  nthw_field_t *mpCcsDataEpp;
  nthw_field_t *mpCcsDataTpeEn;
  nthw_field_t *mpCcsDataTpe;
  nthw_field_t *mpCcsDataRrbEn;
  nthw_field_t *mpCcsDataRrb;
  nthw_field_t *mpCcsDataSb0Type;
  nthw_field_t *mpCcsDataSb0Data;
  nthw_field_t *mpCcsDataSb1Type;
  nthw_field_t *mpCcsDataSb1Data;
  nthw_field_t *mpCcsDataSb2Type;
  nthw_field_t *mpCcsDataSb2Data;
};

#endif /* __FLOW_NTHW_CAT_H__ */
