/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_IOA_H__
#define __FLOW_NTHW_IOA_H__

#include "nthw_fpga_model.h"

#include <stdint.h> /* uint32_t */

struct IoaNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;

  nthw_module_t *mIoa;

  nthw_register_t *mpRcpCtrl;
  nthw_field_t *mpRcpAddr;
  nthw_field_t *mpRcpCnt;
  nthw_register_t *mpRcpData;

  nthw_field_t *mpRcpDataTunnelPop;
  nthw_field_t *mpRcpDataVlanPop;
  nthw_field_t *mpRcpDataVlanPush;
  nthw_field_t *mpRcpDataVlanVid;
  nthw_field_t *mpRcpDataVlanDei;
  nthw_field_t *mpRcpDataVlanPcp;
  nthw_field_t *mpRcpDataVlanTpidSel;
  nthw_field_t *mpRcpDataQueueOverrideEn;
  nthw_field_t *mpRcpDataQueueId;

  nthw_register_t *mpSpecial;
  nthw_field_t *mpSpecialVlanTpidCustTpid0;
  nthw_field_t *mpSpecialVlanTpidCustTpid1;

  nthw_register_t *mpRoaEppCtrl;
  nthw_field_t *mpRoaEppAddr;
  nthw_field_t *mpRoaEppCnt;
  nthw_register_t *mpRoaEppData;
  nthw_field_t *mpRoaEppDataPushTunnel;
  nthw_field_t *mpRoaEppDataTxPort;
};

typedef struct IoaNthw IoaNthw_t;
#if 0
typedef struct IoaNthw IoaNthw;
#endif

struct IoaNthw *IoaNthw_new(void);
void IoaNthw_delete(struct IoaNthw *p);
int IoaNthw_init(struct IoaNthw *p, nthw_fpga_t *pFpga, int nInstance);

int IoaNthw_setup(struct IoaNthw *p, int nIdx, int nIdxCnt);
void ioaNthw_setDebugMode(struct IoaNthw *p, unsigned int nDebugMode);

#if 0
  uint32_t GetVersionMajor(void) {return (mIoa.getVersion()).getMajorVersion();}
  uint32_t GetVersionMinor(void) {return (mIoa.getVersion()).getMinorVersion();}
  void SetDebugOutput(int on) {if (on) mIoa.setDebugMode(Register::ON_WRITE); else mIoa.setDebugMode(Register::NO_DEBUG);}
#endif

/* RCP */
void IoaNthw_RcpSelect(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpCnt(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpTunnelPop(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpVlanPop(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpVlanPush(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpVlanVid(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpVlanDei(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpVlanPcp(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpVlanTpidSel(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpQueueOverrideEn(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpQueueId(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpFlush(const struct IoaNthw *p);

/* Vlan Tpid Special */
void IoaNthw_SpecialVlanTpidCustTpid0(const struct IoaNthw *p, uint32_t val);
void IoaNthw_SpecialVlanTpidCustTpid1(const struct IoaNthw *p, uint32_t val);
void IoaNthw_SpecialVlanTpidFlush(const struct IoaNthw *p);

/* EPP module */
void IoaNthw_RoaEppSelect(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RoaEppCnt(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RoaEppPushTunnel(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RoaEppTxPort(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RoaEppFlush(const struct IoaNthw *p);

#endif /* __FLOW_NTHW_IOA_H__ */
