/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_KM_H__
#define __FLOW_NTHW_KM_H__

#include <stdint.h> /* uint32_t */
#include "nthw_fpga_model.h"

struct KmNthw;

typedef struct KmNthw KmNthw_t;
#if 0
typedef struct KmNthw KmNthw;
#endif

struct KmNthw *KmNthw_new(void);
void KmNthw_delete(struct KmNthw *p);
int KmNthw_init(struct KmNthw *p, nthw_fpga_t *pFpga, int nInstance);

int KmNthw_setup(struct KmNthw *p, int nIdx, int nIdxCnt);
void kmNthw_setDebugMode(struct KmNthw *p, unsigned int nDebugMode);

#if 0
uint32_t GetVersionMajor(void) {return (mKm.getVersion()).getMajorVersion();}
uint32_t GetVersionMinor(void) {return (mKm.getVersion()).getMinorVersion();}
void SetDebugOutput(int on) {if (on) mKm.setDebugMode(Register::ON_WRITE); else mKm.setDebugMode(Register::NO_DEBUG);}
#endif

/* RCP initial v3 */
void KmNthw_RcpSelect(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpCnt(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpQw0Dyn(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpQw0Ofs(const struct KmNthw *p, int32_t val);
void KmNthw_RcpQw0SelA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpQw0SelB(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpQw4Dyn(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpQw4Ofs(const struct KmNthw *p, int32_t val);
void KmNthw_RcpQw4SelA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpQw4SelB(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpSw8Dyn(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpSw8Ofs(const struct KmNthw *p, int32_t val);
void KmNthw_RcpSw8SelA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpSw8SelB(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpSw9Dyn(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpSw9Ofs(const struct KmNthw *p, int32_t val);
void KmNthw_RcpSw9SelA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpSw9SelB(const struct KmNthw *p, uint32_t val);
/* subst in v6 */
void KmNthw_RcpDw8Dyn(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpDw8Ofs(const struct KmNthw *p, int32_t val);
void KmNthw_RcpDw8SelA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpDw8SelB(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpDw10Dyn(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpDw10Ofs(const struct KmNthw *p, int32_t val);
void KmNthw_RcpDw10SelA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpDw10SelB(const struct KmNthw *p, uint32_t val);

void KmNthw_RcpSwxOvsSb(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpSwxCch(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpSwxSelA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpSwxSelB(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpMaskA(const struct KmNthw *p, const uint32_t *val);
void KmNthw_RcpMaskDA(const struct KmNthw *p, const uint32_t *val);
void KmNthw_RcpMaskB(const struct KmNthw *p, const uint32_t *val);
void KmNthw_RcpDual(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpPaired(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpElA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpElB(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpInfoA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpInfoB(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpFtmA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpFtmB(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpBankA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpBankB(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpKlA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpKlB(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpFlowSet(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpKeywayA(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpKeywayB(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpSynergyMode(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpDw0BDyn(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpDw0BOfs(const struct KmNthw *p, int32_t val);
void KmNthw_RcpDw2BDyn(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpDw2BOfs(const struct KmNthw *p, int32_t val);
void KmNthw_RcpSw4BDyn(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpSw4BOfs(const struct KmNthw *p, int32_t val);
void KmNthw_RcpSw5BDyn(const struct KmNthw *p, uint32_t val);
void KmNthw_RcpSw5BOfs(const struct KmNthw *p, int32_t val);
void KmNthw_RcpFlush(const struct KmNthw *p);
/* CAM */
void KmNthw_CamSelect(const struct KmNthw *p, uint32_t val);
void KmNthw_CamCnt(const struct KmNthw *p, uint32_t val);
void KmNthw_CamW0(const struct KmNthw *p, uint32_t val);
void KmNthw_CamW1(const struct KmNthw *p, uint32_t val);
void KmNthw_CamW2(const struct KmNthw *p, uint32_t val);
void KmNthw_CamW3(const struct KmNthw *p, uint32_t val);
void KmNthw_CamW4(const struct KmNthw *p, uint32_t val);
void KmNthw_CamW5(const struct KmNthw *p, uint32_t val);
void KmNthw_CamFt0(const struct KmNthw *p, uint32_t val);
void KmNthw_CamFt1(const struct KmNthw *p, uint32_t val);
void KmNthw_CamFt2(const struct KmNthw *p, uint32_t val);
void KmNthw_CamFt3(const struct KmNthw *p, uint32_t val);
void KmNthw_CamFt4(const struct KmNthw *p, uint32_t val);
void KmNthw_CamFt5(const struct KmNthw *p, uint32_t val);
void KmNthw_CamFlush(const struct KmNthw *p);
/* TCAM */
void KmNthw_TcamSelect(const struct KmNthw *p, uint32_t val);
void KmNthw_TcamCnt(const struct KmNthw *p, uint32_t val);
void KmNthw_TcamT(const struct KmNthw *p, uint32_t *val);
void KmNthw_TcamFlush(const struct KmNthw *p);
/* TCI */
void KmNthw_TciSelect(const struct KmNthw *p, uint32_t val);
void KmNthw_TciCnt(const struct KmNthw *p, uint32_t val);
void KmNthw_TciColor(const struct KmNthw *p, uint32_t val);
void KmNthw_TciFt(const struct KmNthw *p, uint32_t val);
void KmNthw_TciFlush(const struct KmNthw *p);
/* TCQ */
void KmNthw_TcqSelect(const struct KmNthw *p, uint32_t val);
void KmNthw_TcqCnt(const struct KmNthw *p, uint32_t val);
void KmNthw_TcqBankMask(const struct KmNthw *p, uint32_t val);
void KmNthw_TcqQual(const struct KmNthw *p, uint32_t val);
void KmNthw_TcqQual72(const struct KmNthw *p, uint32_t *val);

void KmNthw_TcqFlush(const struct KmNthw *p);

struct KmNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;

  nthw_module_t *mKm;

  nthw_register_t *mpRcpCtrl;
  nthw_field_t *mpRcpAddr;
  nthw_field_t *mpRcpCnt;
  nthw_register_t *mpRcpData;
  nthw_field_t *mpRcpDataQw0Dyn;
  nthw_field_t *mpRcpDataQw0Ofs;
  nthw_field_t *mpRcpDataQw0SelA;
  nthw_field_t *mpRcpDataQw0SelB;
  nthw_field_t *mpRcpDataQw4Dyn;
  nthw_field_t *mpRcpDataQw4Ofs;
  nthw_field_t *mpRcpDataQw4SelA;
  nthw_field_t *mpRcpDataQw4SelB;
  nthw_field_t *mpRcpDataSw8Dyn;
  nthw_field_t *mpRcpDataSw8Ofs;
  nthw_field_t *mpRcpDataSw8SelA;
  nthw_field_t *mpRcpDataSw8SelB;
  nthw_field_t *mpRcpDataSw9Dyn;
  nthw_field_t *mpRcpDataSw9Ofs;
  nthw_field_t *mpRcpDataSw9SelA;
  nthw_field_t *mpRcpDataSw9SelB;

  nthw_field_t *mpRcpDataDw8Dyn;    /* substituted Sw<x> from v6+ */
  nthw_field_t *mpRcpDataDw8Ofs;    /* substituted Sw<x> from v6+ */
  nthw_field_t *mpRcpDataDw8SelA;   /* substituted Sw<x> from v6+ */
  nthw_field_t *mpRcpDataDw8SelB;   /* substituted Sw<x> from v6+ */
  nthw_field_t *mpRcpDataDw10Dyn;   /* substituted Sw<x> from v6+ */
  nthw_field_t *mpRcpDataDw10Ofs;   /* substituted Sw<x> from v6+ */
  nthw_field_t *mpRcpDataDw10SelA;  /* substituted Sw<x> from v6+ */
  nthw_field_t *mpRcpDataDw10SelB;  /* substituted Sw<x> from v6+ */

  nthw_field_t *mpRcpDataSwxOvsSb;
  nthw_field_t *mpRcpDataSwxCch;
  nthw_field_t *mpRcpDataSwxSelA;
  nthw_field_t *mpRcpDataSwxSelB;
  nthw_field_t *mpRcpDataMaskA;
  nthw_field_t *mpRcpDataMaskB;
  nthw_field_t *mpRcpDataDual;
  nthw_field_t *mpRcpDataPaired;
  nthw_field_t *mpRcpDataElA;
  nthw_field_t *mpRcpDataElB;
  nthw_field_t *mpRcpDataInfoA;
  nthw_field_t *mpRcpDataInfoB;
  nthw_field_t *mpRcpDataFtmA;
  nthw_field_t *mpRcpDataFtmB;
  nthw_field_t *mpRcpDataBankA;
  nthw_field_t *mpRcpDataBankB;
  nthw_field_t *mpRcpDataKlA;
  nthw_field_t *mpRcpDataKlB;
  nthw_field_t *mpRcpDataFlowSet;
  nthw_field_t *mpRcpDataKeywayA;
  nthw_field_t *mpRcpDataKeywayB;
  nthw_field_t *mpRcpDataSynergyMode;
  nthw_field_t *mpRcpDataDw0BDyn;
  nthw_field_t *mpRcpDataDw0BOfs;
  nthw_field_t *mpRcpDataDw2BDyn;
  nthw_field_t *mpRcpDataDw2BOfs;
  nthw_field_t *mpRcpDataSw4BDyn;
  nthw_field_t *mpRcpDataSw4BOfs;
  nthw_field_t *mpRcpDataSw5BDyn;
  nthw_field_t *mpRcpDataSw5BOfs;

  nthw_register_t *mpCamCtrl;
  nthw_field_t *mpCamAddr;
  nthw_field_t *mpCamCnt;
  nthw_register_t *mpCamData;
  nthw_field_t *mpCamDataW0;
  nthw_field_t *mpCamDataW1;
  nthw_field_t *mpCamDataW2;
  nthw_field_t *mpCamDataW3;
  nthw_field_t *mpCamDataW4;
  nthw_field_t *mpCamDataW5;
  nthw_field_t *mpCamDataFt0;
  nthw_field_t *mpCamDataFt1;
  nthw_field_t *mpCamDataFt2;
  nthw_field_t *mpCamDataFt3;
  nthw_field_t *mpCamDataFt4;
  nthw_field_t *mpCamDataFt5;

  nthw_register_t *mpTcamCtrl;
  nthw_field_t *mpTcamAddr;
  nthw_field_t *mpTcamCnt;
  nthw_register_t *mpTcamData;
  nthw_field_t *mpTcamDataT;

  nthw_register_t *mpTciCtrl;
  nthw_field_t *mpTciAddr;
  nthw_field_t *mpTciCnt;
  nthw_register_t *mpTciData;
  nthw_field_t *mpTciDataColor;
  nthw_field_t *mpTciDataFt;

  nthw_register_t *mpTcqCtrl;
  nthw_field_t *mpTcqAddr;
  nthw_field_t *mpTcqCnt;
  nthw_register_t *mpTcqData;
  nthw_field_t *mpTcqDataBankMask;
  nthw_field_t *mpTcqDataQual;
};

#endif /* __FLOW_NTHW_KM_H__ */
