/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_QSL_H__
#define __FLOW_NTHW_QSL_H__

#include <stdint.h> /* uint32_t */
#include "nthw_fpga_model.h"

struct QslNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;

  nthw_module_t *mQsl;

  nthw_register_t *mpRcpCtrl;
  nthw_field_t *mpRcpAddr;
  nthw_field_t *mpRcpCnt;
  nthw_register_t *mpRcpData;
  nthw_field_t *mpRcpDataDiscard;
  nthw_field_t *mpRcpDataDrop;
  nthw_field_t *mpRcpDataTblLo;
  nthw_field_t *mpRcpDataTblHi;
  nthw_field_t *mpRcpDataTblIdx;
  nthw_field_t *mpRcpDataTblMsk;
  nthw_field_t *mpRcpDataCao;
  nthw_field_t *mpRcpDataLr;
  nthw_field_t *mpRcpDataTsa;
  nthw_field_t *mpRcpDataVli;

  nthw_register_t *mpLtxCtrl;
  nthw_field_t *mpLtxAddr;
  nthw_field_t *mpLtxCnt;
  nthw_register_t *mpLtxData;
  nthw_field_t *mpLtxDataLr;
  nthw_field_t *mpLtxDataTxPort;
  nthw_field_t *mpLtxDataTsa;

  nthw_register_t *mpQstCtrl;
  nthw_field_t *mpQstAddr;
  nthw_field_t *mpQstCnt;
  nthw_register_t *mpQstData;
  nthw_field_t *mpQstDataQueue;
  nthw_field_t *mpQstDataEn;
  nthw_field_t *mpQstDataTxPort;
  nthw_field_t *mpQstDataLre;
  nthw_field_t *mpQstDataTci;
  nthw_field_t *mpQstDataVen;

  nthw_register_t *mpQenCtrl;
  nthw_field_t *mpQenAddr;
  nthw_field_t *mpQenCnt;
  nthw_register_t *mpQenData;
  nthw_field_t *mpQenDataEn;

  nthw_register_t *mpUnmqCtrl;
  nthw_field_t *mpUnmqAddr;
  nthw_field_t *mpUnmqCnt;
  nthw_register_t *mpUnmqData;
  nthw_field_t *mpUnmqDataDestQueue;
  nthw_field_t *mpUnmqDataEn;
};

typedef struct QslNthw QslNthw_t;
#if 0
typedef struct QslNthw QslNthw;
#endif

struct QslNthw *QslNthw_new(void);
void QslNthw_delete(struct QslNthw *p);
int QslNthw_init(struct QslNthw *p, nthw_fpga_t *pFpga, int nInstance);

int QslNthw_setup(struct QslNthw *p, int nIdx, int nIdxCnt);
void qslNthw_setDebugMode(struct QslNthw *p, unsigned int nDebugMode);

#if 0
uint32_t GetVersionMajor(void) {return (mQsl.getVersion()).getMajorVersion();}
uint32_t GetVersionMinor(void) {return (mQsl.getVersion()).getMinorVersion();}
void SetDebugOutput(int on) {if (on) mQsl.setDebugMode(Register::ON_WRITE); else mQsl.setDebugMode(Register::NO_DEBUG);}
#endif

/* RCP */
void QslNthw_RcpSelect(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpCnt(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpDiscard(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpDrop(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpTblLo(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpTblHi(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpTblIdx(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpTblMsk(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpCao(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpLr(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpTsa(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpVli(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpFlush(const struct QslNthw *p);

/* LTX */
void QslNthw_LtxSelect(const struct QslNthw *p, uint32_t val);
void QslNthw_LtxCnt(const struct QslNthw *p, uint32_t val);
void QslNthw_LtxLr(const struct QslNthw *p, uint32_t val);
void QslNthw_LtxTxPort(const struct QslNthw *p, uint32_t val);
void QslNthw_LtxTsa(const struct QslNthw *p, uint32_t val);
void QslNthw_LtxFlush(const struct QslNthw *p);

/* QST */
void QslNthw_QstSelect(const struct QslNthw *p, uint32_t val);
void QslNthw_QstCnt(const struct QslNthw *p, uint32_t val);
void QslNthw_QstQueue(const struct QslNthw *p, uint32_t val);
void QslNthw_QstEn(const struct QslNthw *p, uint32_t val);
void QslNthw_QstTxPort(const struct QslNthw *p, uint32_t val);
void QslNthw_QstLre(const struct QslNthw *p, uint32_t val);
void QslNthw_QstTci(const struct QslNthw *p, uint32_t val);
void QslNthw_QstVen(const struct QslNthw *p, uint32_t val);
void QslNthw_QstFlush(const struct QslNthw *p);

/* QEN */
void QslNthw_QenSelect(const struct QslNthw *p, uint32_t val);
void QslNthw_QenCnt(const struct QslNthw *p, uint32_t val);
void QslNthw_QenEn(const struct QslNthw *p, uint32_t val);
void QslNthw_QenFlush(const struct QslNthw *p);

/* UNMQ */
void QslNthw_UnmqSelect(const struct QslNthw *p, uint32_t val);
void QslNthw_UnmqCnt(const struct QslNthw *p, uint32_t val);
void QslNthw_UnmqDestQueue(const struct QslNthw *p, uint32_t val);
void QslNthw_UnmqEn(const struct QslNthw *p, uint32_t val);
void QslNthw_UnmqFlush(const struct QslNthw *p);

#endif /* __FLOW_NTHW_QSL_H__ */
