/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_RMC_H__
#define __FLOW_NTHW_RMC_H__

#include <stdint.h> /* uint32_t */
#include "nthw_fpga_model.h"

struct RmcNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;

  nthw_module_t *mRmc;

  nthw_register_t* mpCtrl;
  nthw_field_t* mpCtrlBlockStatt;
  nthw_field_t* mpCtrlBlockKeepA;
  nthw_field_t* mpCtrlBlockRppSlice;
  nthw_field_t* mpCtrlBlockMacPort;
  nthw_field_t* mpCtrlLagPhyOddEven;
};

struct RmcNthw *RmcNthw_new(void);
void RmcNthw_delete(struct RmcNthw *p);
int RmcNthw_init(struct RmcNthw *p, nthw_fpga_t *pFpga, int nInstance);

int RmcNthw_setup(struct RmcNthw *p, int nIdx, int nIdxCnt);
void rmcNthw_setDebugMode(struct RmcNthw *p, unsigned int nDebugMode);

#if 0
  uint32_t GetVersionMajor(void) {return (mRmc.getVersion()).getMajorVersion();}
  uint32_t GetVersionMinor(void) {return (mRmc.getVersion()).getMinorVersion();}
  void SetDebugOutput(int on) {if (on) mRmc.setDebugMode(Register::ON_WRITE); else mRmc.setDebugMode(Register::NO_DEBUG);}
#endif

/* CTRL */
void RmcNthw_CtrlBlockStatt(const struct RmcNthw *p, uint32_t val);
void RmcNthw_CtrlBlockKeepA(const struct RmcNthw *p, uint32_t val);
void RmcNthw_CtrlBlockRppSlice(const struct RmcNthw *p, uint32_t val);
void RmcNthw_CtrlBlockMacPort(const struct RmcNthw *p, uint32_t val);
void RmcNthw_CtrlLagPhyOddEven(const struct RmcNthw *p, uint32_t val);
void RmcNthw_CtrlFlush(const struct RmcNthw *p);

#endif /* __FLOW_NTHW_RMC_H__ */
