/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_TX_RPL_H__
#define __FLOW_NTHW_TX_RPL_H__

#include <stdint.h>
#include "nthw_fpga_model.h"

struct TxRplNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;

  nthw_module_t *mTxRpl;

  nthw_register_t *mpRcpCtrl;
  nthw_field_t *mpRcpCtrlAddr;
  nthw_field_t *mpRcpCtrlCnt;

  nthw_register_t *mpRcpData;
  nthw_field_t *mpRcpDataDyn;
  nthw_field_t *mpRcpDataOfs;
  nthw_field_t *mpRcpDataLen;
  nthw_field_t *mpRcpDataRplPtr;
  nthw_field_t *mpRcpDataExtPrio;
  nthw_field_t *mpRcpDataEthTypeWr;

  nthw_register_t *mpExtCtrl;
  nthw_field_t *mpExtCtrlAddr;
  nthw_field_t *mpExtCtrlCnt;

  nthw_register_t *mpExtData;
  nthw_field_t *mpExtDataRplPtr;

  nthw_register_t *mpRplCtrl;
  nthw_field_t *mpRplCtrlAddr;
  nthw_field_t *mpRplCtrlCnt;

  nthw_register_t *mpRplData;
  nthw_field_t *mpRplDataValue;
};

struct TxRplNthw *TxRplNthw_new(void);
void TxRplNthw_delete(struct TxRplNthw *p);
int TxRplNthw_init(struct TxRplNthw *p, nthw_fpga_t *pFpga, int nInstance);

int TxRplNthw_setup(struct TxRplNthw *p, int nIdx, int nIdxCnt);
void txRplNthw_setDebugMode(struct TxRplNthw *p, unsigned int nDebugMode);

#if 0
  uint32_t GetVersionMajor(void) {return (mTxRpl.getVersion()).getMajorVersion();}
  uint32_t GetVersionMinor(void) {return (mTxRpl.getVersion()).getMinorVersion();}
  void SetDebugOutput(int on) {if (on) mTxRpl.setDebugMode(Register::ON_WRITE); else mTxRpl.setDebugMode(Register::NO_DEBUG);}
#endif

/* RCP */
void TxRplNthw_RcpSelect(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_RcpCnt(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_RcpDyn(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_RcpOfs(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_RcpLen(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_RcpRplPtr(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_RcpExtPrio(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_RcpEthTypeWr(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_RcpFlush(const struct TxRplNthw *p);

void TxRplNthw_ExtSelect(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_ExtCnt(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_ExtRplPtr(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_ExtFlush(const struct TxRplNthw *p);

void TxRplNthw_RplSelect(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_RplCnt(const struct TxRplNthw *p, uint32_t val);
void TxRplNthw_RplValue(const struct TxRplNthw *p, const uint32_t* val);
void TxRplNthw_RplFlush(const struct TxRplNthw *p);

#endif /* __FLOW_NTHW_TX_RPL_H__ */
