/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_DBS_HPP_
#define NTHW_DBS_HPP_

#if 0
/* Note: for now defined here. Ideally get from P200 file */
#endif
#define NT_DBS_RX_QUEUES_MAX (128)
#define NT_DBS_TX_QUEUES_MAX (128)

/*
 * Struct for implementation of memory bank shadows
 */

/* DBS_RX_AM_DATA */
struct NthwDbsRxAmData_s
{
  uint64_t guestPhysicalAddress;
  uint32_t enable;
  uint32_t hostId;
  uint32_t packed;
  uint32_t int_enable;
};

/* DBS_TX_AM_DATA */
struct NthwDbsTxAmData_s
{
  uint64_t guestPhysicalAddress;
  uint32_t enable;
  uint32_t hostId;
  uint32_t packed;
  uint32_t int_enable;
};

/* DBS_RX_UW_DATA */
struct NthwDbsRxUwData_s
{
  uint64_t guestPhysicalAddress;
  uint32_t hostId;
  uint32_t queueSize;
  uint32_t packed;
  uint32_t int_enable;
  uint32_t vec;
  uint32_t istk;
};

/* DBS_TX_UW_DATA */
struct NthwDbsTxUwData_s
{
  uint64_t guestPhysicalAddress;
  uint32_t hostId;
  uint32_t queueSize;
  uint32_t packed;
  uint32_t int_enable;
  uint32_t vec;
  uint32_t istk;
  uint32_t inOrder;
};

/* DBS_RX_DR_DATA */
struct NthwDbsRxDrData_s
{
  uint64_t guestPhysicalAddress;
  uint32_t hostId;
  uint32_t queueSize;
  uint32_t header;
  uint32_t packed;
};

/* DBS_TX_DR_DATA */
struct NthwDbsTxDrData_s
{
  uint64_t guestPhysicalAddress;
  uint32_t hostId;
  uint32_t queueSize;
  uint32_t header;
  uint32_t port;
  uint32_t packed;
};

/* DBS_TX_QP_DATA */
struct NthwDbsTxQpData_s
{
  uint32_t virtualPort;
};

struct NthwDbsTxQosData_s
{
  uint32_t enable;
  uint32_t ir;
  uint32_t bs;
};


struct NthwDbs_s
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModDbs;
  int mnInstance;

  int mnParamDbsPresent;

  nthw_register_t* mpRegRxControl;
  nthw_field_t* mpFldRxControlLastQueue;
  nthw_field_t* mpFldRxControlAvailMonitorEnable;
  nthw_field_t* mpFldRxControlAvailMonitorScanSpeed;
  nthw_field_t* mpFldRxControlUsedWriteEnable;
  nthw_field_t* mpFldRxControlUsedWriterUpdateSpeed;
  nthw_field_t* mpFldRxControlRxQueuesEnable;

  nthw_register_t* mpRegTxControl;
  nthw_field_t* mpFldTxControlLastQueue;
  nthw_field_t* mpFldTxControlAvailMonitorEnable;
  nthw_field_t* mpFldTxControlAvailMonitorScanSpeed;
  nthw_field_t* mpFldTxControlUsedWriteEnable;
  nthw_field_t* mpFldTxControlUsedWriterUpdateSpeed;
  nthw_field_t* mpFldTxControlTxQueuesEnable;

  nthw_register_t* mpRegRxInit;
  nthw_field_t* mpFldRxInitInit;
  nthw_field_t* mpFldRxInitQueue;
  nthw_field_t* mpFldRxInitBusy;

  nthw_register_t* mpRegRxInitVal;
  nthw_field_t* mpFldRxInitValIdx;
  nthw_field_t* mpFldRxInitValPtr;

  nthw_register_t* mpRegRxPtr;
  nthw_field_t* mpFldRxPtrPtr;
  nthw_field_t* mpFldRxPtrQueue;
  nthw_field_t* mpFldRxPtrValid;

  nthw_register_t* mpRegTxInit;
  nthw_field_t* mpFldTxInitInit;
  nthw_field_t* mpFldTxInitQueue;
  nthw_field_t* mpFldTxInitBusy;

  nthw_register_t* mpRegTxInitVal;
  nthw_field_t* mpFldTxInitValIdx;
  nthw_field_t* mpFldTxInitValPtr;

  nthw_register_t* mpRegTxPtr;
  nthw_field_t* mpFldTxPtrPtr;
  nthw_field_t* mpFldTxPtrQueue;
  nthw_field_t* mpFldTxPtrValid;

  nthw_register_t* mpRegRxIdle;
  nthw_field_t* mpFldRxIdleIdle;
  nthw_field_t* mpFldRxIdleQueue;
  nthw_field_t* mpFldRxIdleBusy;

  nthw_register_t* mpRegTxIdle;
  nthw_field_t* mpFldTxIdleIdle;
  nthw_field_t* mpFldTxIdleQueue;
  nthw_field_t* mpFldTxIdleBusy;

  nthw_register_t* mpRegRxAvailMonitorControl;
  nthw_field_t* mpFldRxAvailMonitorControlAdr;
  nthw_field_t* mpFldRxAvailMonitorControlCnt;

  nthw_register_t* mpRegRxAvailMonitorData;
  nthw_field_t* mpFldRxAvailMonitorDataGuestPhysicalAddress;
  nthw_field_t* mpFldRxAvailMonitorDataEnable;
  nthw_field_t* mpFldRxAvailMonitorDataHostID;
  nthw_field_t* mpFldRxAvailMonitorDataPacked;
  nthw_field_t* mpFldRxAvailMonitorDataInt;

  nthw_register_t* mpRegTxAvailMonitorControl;
  nthw_field_t* mpFldTxAvailMonitorControlAdr;
  nthw_field_t* mpFldTxAvailMonitorControlCnt;

  nthw_register_t* mpRegTxAvailMonitorData;
  nthw_field_t* mpFldTxAvailMonitorDataGuestPhysicalAddress;
  nthw_field_t* mpFldTxAvailMonitorDataEnable;
  nthw_field_t* mpFldTxAvailMonitorDataHostID;
  nthw_field_t* mpFldTxAvailMonitorDataPacked;
  nthw_field_t* mpFldTxAvailMonitorDataInt;

  nthw_register_t* mpRegRxUsedWriterControl;
  nthw_field_t* mpFldRxUsedWriterControlAdr;
  nthw_field_t* mpFldRxUsedWriterControlCnt;

  nthw_register_t* mpRegRxUsedWriterData;
  nthw_field_t* mpFldRxUsedWriterDataGuestPhysicalAddress;
  nthw_field_t* mpFldRxUsedWriterDataHostID;
  nthw_field_t* mpFldRxUsedWriterDataQueueSize;
  nthw_field_t* mpFldRxUsedWriterDataPacked;
  nthw_field_t* mpFldRxUsedWriterDataInt;
  nthw_field_t* mpFldRxUsedWriterDataVec;
  nthw_field_t* mpFldRxUsedWriterDataIstk;

  nthw_register_t* mpRegTxUsedWriterControl;
  nthw_field_t* mpFldTxUsedWriterControlAdr;
  nthw_field_t* mpFldTxUsedWriterControlCnt;

  nthw_register_t* mpRegTxUsedWriterData;
  nthw_field_t* mpFldTxUsedWriterDataGuestPhysicalAddress;
  nthw_field_t* mpFldTxUsedWriterDataHostID;
  nthw_field_t* mpFldTxUsedWriterDataQueueSize;
  nthw_field_t* mpFldTxUsedWriterDataPacked;
  nthw_field_t* mpFldTxUsedWriterDataInt;
  nthw_field_t* mpFldTxUsedWriterDataVec;
  nthw_field_t* mpFldTxUsedWriterDataIstk;
  nthw_field_t* mpFldTxUsedWriterDataInOrder;

  nthw_register_t* mpRegRxDescriptorReaderControl;
  nthw_field_t* mpFldRxDescriptorReaderControlAdr;
  nthw_field_t* mpFldRxDescriptorReaderControlCnt;

  nthw_register_t* mpRegRxDescriptorReaderData;
  nthw_field_t* mpFldRxDescriptorReaderDataGuestPhysicalAddress;
  nthw_field_t* mpFldRxDescriptorReaderDataHostID;
  nthw_field_t* mpFldRxDescriptorReaderDataQueueSize;
  nthw_field_t* mpFldRxDescriptorReaderDataHeader;
  nthw_field_t* mpFldRxDescriptorReaderDataPacked;

  nthw_register_t* mpRegTxDescriptorReaderControl;
  nthw_field_t* mpFldTxDescriptorReaderControlAdr;
  nthw_field_t* mpFldTxDescriptorReaderControlCnt;

  nthw_register_t* mpRegTxDescriptorReaderData;
  nthw_field_t* mpFldTxDescriptorReaderDataGuestPhysicalAddress;
  nthw_field_t* mpFldTxDescriptorReaderDataHostID;
  nthw_field_t* mpFldTxDescriptorReaderDataQueueSize;
  nthw_field_t* mpFldTxDescriptorReaderDataPort;
  nthw_field_t* mpFldTxDescriptorReaderDataHeader;
  nthw_field_t* mpFldTxDescriptorReaderDataPacked;

  nthw_register_t* mpRegTxQueuePropertyControl;
  nthw_field_t* mpFldTxQueuePropertyControlAdr;
  nthw_field_t* mpFldTxQueuePropertyControlCnt;

  nthw_register_t* mpRegTxQueuePropertyData;
  nthw_field_t* mpFldTxQueuePropertyDataVPort;

  nthw_register_t* mpRegTxQueueQosControl;
  nthw_field_t* mpRegTxQueueQosControlAdr;
  nthw_field_t* mpRegTxQueueQosControlCnt;

  nthw_register_t* mpRegTxQueueQosData;
  nthw_field_t* mpRegTxQueueQosDataEn;
  nthw_field_t* mpRegTxQueueQosDataIR;
  nthw_field_t* mpRegTxQueueQosDataBS;

  nthw_register_t* mpRegTxQueueQosRate;
  nthw_field_t* mpRegTxQueueQosRateMul;
  nthw_field_t* mpRegTxQueueQosRateDiv;

  struct NthwDbsRxAmData_s mRxAmShadow[NT_DBS_RX_QUEUES_MAX];
  struct NthwDbsRxUwData_s mRxUwShadow[NT_DBS_RX_QUEUES_MAX];
  struct NthwDbsRxDrData_s mRxDrShadow[NT_DBS_RX_QUEUES_MAX];

  struct NthwDbsTxAmData_s mTxAmShadow[NT_DBS_TX_QUEUES_MAX];
  struct NthwDbsTxUwData_s mTxUwShadow[NT_DBS_TX_QUEUES_MAX];
  struct NthwDbsTxDrData_s mTxDrShadow[NT_DBS_TX_QUEUES_MAX];
  struct NthwDbsTxQpData_s mTxQpShadow[NT_DBS_TX_QUEUES_MAX];
  struct NthwDbsTxQosData_s mTxQosShadow[NT_DBS_TX_QUEUES_MAX];
};

typedef struct NthwDbs_s NthwDbs_t;

NthwDbs_t* NthwDbs_new(void);
void NthwDbs_delete(NthwDbs_t* p);
int dbs_init(NthwDbs_t* p, nthw_fpga_t* pFpga, int nInstance);
void dbs_reset(NthwDbs_t* p);

int dbs_reset_rx_control(NthwDbs_t* p);
int dbs_reset_tx_control(NthwDbs_t* p);
int set_rx_control(NthwDbs_t* p,
                         uint32_t lastQueue,
                         uint32_t availMonitorEnable,
                         uint32_t availMonitorSpeed,
                         uint32_t usedWriteEnable,
                         uint32_t usedWriteSpeed,
                         uint32_t rxQueueEnable);
int NthwDbs_getRxControl(NthwDbs_t* p,
                         uint32_t *lastQueue,
                         uint32_t *availMonitorEnable,
                         uint32_t *availMonitorSpeed,
                         uint32_t *usedWriteEnable,
                         uint32_t *usedWriteSpeed,
                         uint32_t *rxQueueEnable);
int set_tx_control(NthwDbs_t* p,
                         uint32_t lastQueue,
                         uint32_t availMonitorEnable,
                         uint32_t availMonitorSpeed,
                         uint32_t usedWriteEnable,
                         uint32_t usedWriteSpeed,
                         uint32_t txQueueEnable);
int NthwDbs_getTxControl(NthwDbs_t* p,
                         uint32_t *lastQueue,
                         uint32_t *availMonitorEnable,
                         uint32_t *availMonitorSpeed,
                         uint32_t *usedWriteEnable,
                         uint32_t *usedWriteSpeed,
                         uint32_t *txQueueEnable);
int set_rx_init(NthwDbs_t* p,
                      uint32_t startIdx,
                      uint32_t startPtr,
                      uint32_t init,
                      uint32_t queue);
int get_rx_init(NthwDbs_t* p,
                      uint32_t *init,
                      uint32_t *queue,
                      uint32_t *busy);
int set_tx_init(NthwDbs_t* p,
                      uint32_t startIdx,
                      uint32_t startPtr,
                      uint32_t init,
                      uint32_t queue);
int get_tx_init(NthwDbs_t* p,
                      uint32_t *init,
                      uint32_t *queue,
                      uint32_t *busy);
int set_rx_idle(NthwDbs_t* p,
                      uint32_t idle,
                      uint32_t queue);
int get_rx_idle(NthwDbs_t* p,
                      uint32_t *idle,
                      uint32_t *queue,
                      uint32_t *busy);
int set_tx_idle(NthwDbs_t* p,
                      uint32_t idle,
                      uint32_t queue);
int get_tx_idle(NthwDbs_t* p,
                      uint32_t *idle,
                      uint32_t *queue,
                      uint32_t *busy);
int set_rx_ptr_queue(NthwDbs_t* p, uint32_t queue);
int get_rx_ptr(NthwDbs_t* p,
                     uint32_t *ptr,
                     uint32_t *queue,
                     uint32_t *valid);
int set_tx_ptr_queue(NthwDbs_t* p, uint32_t queue);
int get_tx_ptr(NthwDbs_t* p,
                     uint32_t *ptr,
                     uint32_t *queue,
                     uint32_t *valid);
int set_rx_am_data(NthwDbs_t* p,
                        uint32_t index,
                        uint64_t guestPhysicalAddress,
                        uint32_t enable,
                        uint32_t hostId,
                        uint32_t packed,
                        uint32_t int_enable);
int set_tx_am_data(NthwDbs_t* p,
                        uint32_t index,
                        uint64_t guestPhysicalAddress,
                        uint32_t enable,
                        uint32_t hostId,
                        uint32_t packed,
                        uint32_t int_enable);
int set_rx_uw_data(NthwDbs_t* p,
                        uint32_t index,
                        uint64_t guestPhysicalAddress,
                        uint32_t hostId,
                        uint32_t queueSize,
                        uint32_t packed,
                        uint32_t int_enable,
                        uint32_t vec,
                        uint32_t istk);
int set_tx_uw_data(NthwDbs_t* p,
                        uint32_t index,
                        uint64_t guestPhysicalAddress,
                        uint32_t hostId,
                        uint32_t queueSize,
                        uint32_t packed,
                        uint32_t int_enable,
                        uint32_t vec,
                        uint32_t istk,
                        uint32_t inOrder);
int set_rx_dr_data(NthwDbs_t* p,
                        uint32_t index,
                        uint64_t guestPhysicalAddress,
                        uint32_t hostId,
                        uint32_t queueSize,
                        uint32_t header,
                        uint32_t packed);
int set_tx_dr_data(NthwDbs_t* p,
                        uint32_t index,
                        uint64_t guestPhysicalAddress,
                        uint32_t hostId,
                        uint32_t queueSize,
                        uint32_t port,
                        uint32_t header,
                        uint32_t packed);
int NthwDbs_setTxQpData(NthwDbs_t* p,
                        uint32_t index,
                        uint32_t virtualPort);
int set_tx_qos_data(NthwDbs_t* p,
                         uint32_t index,
                         uint32_t enable,
                         uint32_t ir,
                         uint32_t bs);
int set_tx_qos_rate(NthwDbs_t* p,
                         uint32_t mul,
                         uint32_t div);

#endif /* NTHW_DBS_H_ */
