/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_MCU_H_
#define NTHW_MCU_H_

struct NthwMcu
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModMcu;
  int mnInstance;

  int mnModuleMajorVersion;
  int mnModuleMinorVersion;

  uint64_t mnModuleVersionPacked64;

  bool mb_McuPresent;
  int mn_McuType;
  int mn_McuDramSize;

  nthw_register_t* mpRegCsr;
  nthw_field_t* mpFldCsrHalt;
  nthw_field_t* mpFldCsrPause;
  nthw_field_t* mpFldCsrReset;

  nthw_register_t* mpRegDramCtrl;
  nthw_field_t* mpFldDramCtrlAdr;
  nthw_field_t* mpFldDramCtrlCnt;

  nthw_register_t* mpRegDramRdData;
  nthw_field_t* mpFldDramRdDataData;

  nthw_register_t* mpRegDramWrData;
  nthw_field_t* mpFldDramWrDataData;

  nthw_register_t* mpRegIramCtrl;
  nthw_field_t* mpFldIramCtrlAdr;
  nthw_field_t* mpFldIramCtrlCnt;

  nthw_register_t* mpRegIramData;
  nthw_field_t* mpFldIramDataData;
};

typedef struct NthwMcu NthwMcu_t;
typedef struct NthwMcu NthwMcu;

NthwMcu_t* NthwMcu_new(void);
void NthwMcu_delete(NthwMcu_t* p);
int NthwMcu_init(NthwMcu_t *p, nthw_fpga_t *pFpga, int nInstance);

void NthwMcu_setHalt(NthwMcu_t* p, uint32_t value);
uint32_t NthwMcu_getHalt(NthwMcu_t* p);

void NthwMcu_setPause(NthwMcu_t* p, uint32_t value);
uint32_t NthwMcu_getPause(NthwMcu_t* p);

void NthwMcu_setReset(NthwMcu_t* p, uint32_t value);
uint32_t NthwMcu_getReset(NthwMcu_t* p);

int NthwMcu_DramRead(NthwMcu_t *p, uint32_t nAddr, uint8_t* pData, int nDataCnt);
int NthwMcu_DramWrite(NthwMcu_t *p, uint32_t nAddr, uint8_t* pData, int nDataCnt);

int NthwMcu_IramRead(NthwMcu_t *p, uint32_t nAddr, uint8_t* pData, int nDataCnt);
int NthwMcu_IramWrite(NthwMcu_t *p, uint32_t nAddr, uint8_t* pData, int nDataCnt);

#endif /* NTHW_MCU_H_ */
