/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#include "ntavr.h"
#include "ntlog.h"

static int txrx(Nthw_Spi_V3_t *s_spi,
                enum avr_opcodes opcode,
                size_t txsz,
                uint16_t *_tx,
                size_t *rxsz,
                uint16_t *_rx)
{
    int res = 1;
    struct TxRxBuf tx = {.size = (uint16_t)txsz, .pBuf = _tx};
    struct TxRxBuf rx = {.size = (uint16_t)*rxsz, .pBuf = _rx};

    res = Nthw_Spi_V3_transfer(s_spi, opcode, &tx, &rx);
    if(res) {
        NT_LOG(ERR, ETHDEV, "%s transfer failed - %i", __func__, res);
        return res;
    }

    if (rxsz != NULL) {*rxsz = rx.size;}

    return res;
}

uint32_t sensor_read(NthwSpis_t* t_spi, uint8_t fpgaIdx, uint32_t *pSensorResult)
{
    return NthwSpis_read_sensor(t_spi, fpgaIdx, pSensorResult);
}

int NtAvrSensorMonSetup(struct sensor_mon_setup16* pSetup, Nthw_Spi_V3_t *s_spi)
{
    int error;
    size_t  TxSize;
    size_t  RxSize = 0;

    TxSize = sizeof(struct sensor_mon_setup16) - sizeof(pSetup->setup_data);
    TxSize += sizeof(pSetup->setup_data[0]) * pSetup->setup_cnt;

    error = txrx(s_spi, AVR_OP_SENSOR_MON_SETUP, TxSize, (uint16_t*)pSetup, &RxSize, NULL);

    if(error) {
        NT_LOG(ERR, ETHDEV, "%s failed\n", __func__);
        return error;
    }

    if (RxSize != 0)
    {
        NT_LOG(ERR, ETHDEV, "%s: Returned data: Expected size = 0, Actual = %ld", __func__, RxSize);
        return 1;
    }
    return 0;
}

int NtAvrSensorMonCtrl(Nthw_Spi_V3_t *s_spi, enum sensor_mon_control ctrl)
{
    int error;
    size_t RxSize = 0;

    error = txrx(s_spi, AVR_OP_SENSOR_MON_CONTROL, sizeof(ctrl), (uint16_t*)(&ctrl), &RxSize, NULL);

    if (error != 0) {return error;}

    if (RxSize != 0)
    {
        NT_LOG(ERR, ETHDEV, "%s: Returned data: Expected size = 0, Actual = %ld", __func__, RxSize);
        return 1;
    }

    return 0;
}
