# 1. Forward DPDK application

The purpose of this section is to outline
- what demo application is and how to run it,
- topology description required for run the application,
- supported feature set,
- how to run application and which results it generates,

## 1.1 General Description

Forward DPDK application - it is a DPDK demo application for demonstration
Napatech smartNIC deals with traffic forwarding using RTE flow API.

Forward DPDK application has two main features:
* HW offloading with traffic learning.
* Packet forwarding executed with/without encapsulation.

Forward DPDK application forward traffic from ingress to the egress port
without encaplusation by default. However packet encapsulation can be
configured with the appropriate options.

When such a packet is received on the egress port the application will program a flow with
actions to decapsulate the Eth, IP, outer UDP and GTP headers and
encapsulate a new Eth header. Further the packet will be transmitted
on egress port.

Port 1 and port 0 can be programmed to ingress and egress role accordingly.

Once the flows are programmed all packet processing is handled
by HW and packets belonging to the flows are no longer sent to
the application.

Forward DPDK application supports next feature set:

* Setting ingress/egress ports
* Stting VLANs for ingress/egress ports
* Default flows learning
* Disable filters setting
* Disable hardware offload
* Meter flow statistics
* `GTP-U` (GPRS Tunneling Protocol user plane) encapsulation and decapsulation for IPv4 and IPv6
* `CIR,PIR,CBS,PBS` RFC2698 Peak Information Rate limit for RX stream
* `RSS` (Receive Side Scaling). Allow to distribute RX stream between multiple CPUs and queues.


To check the metrics and counters you can use NT tools: `monitoring` and
`statistics`.
For detailed counters description please refer to 1.4 section.

1.2 Topology Description

The Forward DPDK setup consists of two Nodes:

* Node 1 - software or hardware traffic generator capable
  to generate traffic with GTPv1U headers

* Node 2 - Server with running Forward DPDK application

>NOTE: Node 2 must have installed Napatech SmartNIC

Nodes are connected between each other.

```
        Node 1                                           Node 2
  +--------------------+                         +----------------------+
  |           |        |                         |          |           |
  |  traffic  | port 0 |<----------------------->| port 0   |  Forward  |
  |           | ______ |                         |          |   DPDK    |
  | generator |        |                         |          |application|
  |           | port 1 |<----------------------->| port 1   |           |
  |           | ______ |                         |          |           |
  +--------------------+                         +----------------------+
```
In case packet decap is disabled and port 0 is ingress port:
  __Ports dedications for Node 1__

  Port 0 - Transmits with GTPv1U header - Eth/VLAN=44/IPv4/UDP/GTP/IPv4/TCP
  Port 1 - Receives with GTPv1 header   - Eth/VLAN=44/IPv4/UDP/GTP/IPv4/TCP

  __Ports dedications for Node 2__

  Port 0      - Receives packets with GTPv1U header
  Forward App - Forward to the Port 1
  Port 1      - Transmits packets with GTPv1U header


In case packet decap is enabled and port 0 is ingress port:
  __Ports dedications for Node 1__

  Port 0 - Transmits with GTPv1U header  - Eth/VLAN=44/IPv4/UDP/GTP/IPv4/TCP
  Port 1 - Receives without GTPv1 header - Eth/VLAN=45/IPv4/TCP

  __Ports dedications for Node 2__

  Port 0      - Receives packets with GTPv1U header
  Forward App - Encapsulate packet from GTPv1U header - Forward to the Port 1
  Port 1      - Transmits packets withouth GTPv1U header

## 1.4 Statistics

Using the `-s` command line option the application can print statistics for a range of
meter IDs.

To further check counters and metrics user can use `monitoring` and `statistics` tools.
Both of these tools allow to check: RX/TX counters, queues counters and
flow matcher counters(FLM).
Additionally monitoring tool allows to verify PPS and bitrate.

Please follow build instructions at section 2. below. Please note, that DPDK with NT
patch must be built before NT tools.

>NOTE: NT main application with `ntnic` PMD (e.g. Forward DPDK application) must be up and running,
 before execution of NT tools.

### 1.4.1 How to start monitoring tool

  Please run following commands from the root directory of extracted NT inline package:
  ```bash
  cd tools/ntnic-tools/build/monitoring
  ./monitoring
  ```

### 1.4.2 How to start statistics tool

  Please run following commands from the root directory of extracted NT inline package:
  ```bash
  cd tools/ntnic-tools/build/statistic
  ```

  - Check TX/RX statistics (Port counters)

  ```bash
  ./statistic -p 0 -g txrx_stats
  ```

  - Check Queue statistics (Queues counters)

  ```bash
  ./statistic -p 0 -g que_stats
  ```

  - Check Color statistics

  ```bash
  ./statistic -p 0 -g color_stats
  ```

  - Check flow matcher statistics

  ```bash
  ./statistic -p 0 -g flm_stats
  ```


# 2. Environment How-to instructions (Node 2)

This section includes detailed instructions on how to build,
configure and run UPF application for Node 2.

## 2.1 Get NT inline release package

  Please download and extract official release package of NT Inline product.
  The Forward DPDK application and NT tools are part of this package as shown below:
  ```
  |-- dpdk_v22.11.1
  |   `-- dpdk_v22.11.1_*_1.0.0.*
  |-- examples_v22.11.1
  |       |-- examples
  |       |   |-- forward
  |       |   |-- inline_upf_demo
  ...
  `-- tools
      `-- ntnic-tools
  ```
  >NOTE: Exact version and hash number at DPDK directory name is replaced by bash wildcard.

## 2.2 Build DPDK + NTNIC

  In the project root, configure DPDK to be built with debug.
  Output directory is `build`.

  ```bash
  cd dpdk_v22.11.1/dpdk_v22.11.1_*
  meson --buildtype=debug build
  ```

  Omit `--buildtype=debug` if debug is not wanted.

  In order to build and install DPDK libraries, run:

  ```bash
  ninja -C build
  ninja -C build install
  ```

  Add `/usr/local/lib` and the path to the DPDK libraries to the `/etc/ld.so.conf.d/dpdk.conf` file.
  Note: This step applies only to some Linux distributions, such as CentOS, Fedora or RHEL where
  paths in `/usr/local` are not in the default path for the dynamic loader.

  ```bash
  echo -e "/usr/local/lib64" >> /etc/ld.so.conf.d/dpdk.conf
  echo -e "/usr/local/lib64/dpdk" >> /etc/ld.so.conf.d/dpdk.conf
  ldconfig
  ```

## 2.3 Build forward app

  Please run following commands from the root directory of extracted NT inline package:

  ```bash
  export NT_DPDK_PATH=$PWD/dpdk_v22.11.1/dpdk_v22.11.1_*
  export PKG_CONFIG_PATH=/usr/local/lib64/pkgconfig
  cd examples_v22.11.1/examples/forward
  meson build
  ninja -C build
  ```

## 2.4 Build nt_tools to use statistics, monitoring and sensors tools

  Please run following commands from the root directory of extracted NT inline package:

  ```bash
  export NT_DPDK_PATH=$PWD/dpdk_v22.11.1/dpdk_v22.11.1_*
  cd tools/ntnic-tools
  meson build
  ninja -C build
  ```

## 2.5 Prerequisites to run

### 2.5.1 Add hugepages

  The reservation of hugepages can be performed at run time.
  This is done by echoing the number of hugepages required
  to a `nr_hugepages` file in the `/sys/kernel/` directory corresponding
  to a specific page size (in Kilobytes). For a single-node system,
  the command to use is as follows
  (assuming that 32 of 1 GB pages are required):

  ```bash
  echo 32 > /sys/kernel/mm/hugepages/hugepages-1048576kB/nr_hugepages
  ```

  To make the hugepages of size 1 GB available for DPDK use,
  following steps must be performed:

  ```bash
  mkdir /mnt/huge
  mount -t hugetlbfs pagesize=1GB /mnt/huge
  ```

### 2.5.2 Install supported FPGA

  Please install FPGA image delivered with NT inline package. The image is available at `images` directory.
  The Forward DPDK application won't work correctly if unsupported FPGA image version is used.

### 2.5.3 Load the vfio-pci driver with the command

  ```bash
  modprobe vfio-pci enable_sriov=1
  ```

### 2.5.4 Bind device to DPDK

  Find the PCI id for the Napatech adapter by running the following command
   from the DPDK root:

  ```bash
  ./usertools/dpdk-devbind.py  --status
  ```

  ```log
  Other Network devices
  =====================
  0000:10:00.0 'NT200A02 Network Adapter 01c5' unused=vfio-pci
  ```

  In our case we want to use the adapter `NT200A02 Network Adapter 01c5`.
  The PCI id is `0000:10:00.0`.

  Use the PCI id to bind the vfio-pci driver to the adapter
  using the following command:

  ```bash
  ./usertools/dpdk-devbind.py -b vfio-pci 0000:10:00.0
  ```

## 2.6 How to start Forward DPDK application

Forward DPDK application runs as common DPDK application with only one
mandatory argument (allowed PCI id)

```bash
cd dpdk_v22.11.1/dpdk_v22.11.1_*/build/examples
./forward
```

Syntax:
```bash
./forward [-h][-i port_num][-e port_num][-I vlan_id][-E vlan_id][-c][-s][-n][-q][-s][-m profile_value][-l][-d]

```

### 2.6.1 Internal CLI arguments for configuration

#### 2.6.1.1 Help menu
Use `-h` command to display help menu with information, mentioned above

Syntax:
```bash
-h
```

Usage example:
```bash
./forward -- -h
```
If other internal options are specified along with this parameter, they are ignored.

#### 2.6.1.2 Ingress port
Use `-i` command to set ingress direction with specific NIC port

Syntax:
```bash
-i <port_num>
```

Usage example:
```bash
./forward -- -i 0
```

#### 2.6.1.3 Egress port
Use `-i` command to set egress direction with specific NIC port

Syntax:
```bash
-i <port_num>
```

Usage example:
```bash
./forward -- -i 1
```
#### 2.6.1.4 Ingress port VLAN
Use `-I` command to set ingress VLAN with specific NIC port

Syntax:
```bash
-I <vlan_id>
```

Usage example:
```bash
./forward -- -I 35
```

Note: Default VLAN is port 44.

#### 2.6.1.5 Egress port VLAN
Use `-I` command to set egress VLAN with specific NIC port

Syntax:
```bash
-E <vlan_id>
```

Usage example:
```bash
./forward -- -E 56
```

Note: Default VLAN is port 45.

#### 2.6.1.6 Decap and encap packets
Use `-c` command to decap and encap packets befor forwarding between ingress and egress ports.

Syntax:
```bash
-c
```

Usage example:
```bash
./forward -- -c
```

#### 2.6.1.7 Disable Hardware offloading
Use `-n` command to disable hardware offloading. This option will disable flow learning

Syntax:
```bash
-n
```

Usage example:
```bash
./forward -- -n
```

Note: Default HW is enabled

#### 2.6.1.8 Extended statistics
Use `-s` command to show extended flow, rx, tx counters' statistics.

Syntax:
```bash
-s
```

Usage example:
```bash
./forward -- -s
```

#### 2.6.1.9 Meter flow statistics
Use the -m command to set the metering and policing profile with a specific value. This profile determines the allowed average rate of traffic in bits per second based on the user-provided value.

Syntax:
```bash
-m <profile_value>
```
<profile_value>: An integer value representing the metering and policing profile in Gib. This value is used to configure the profile's Committed Information Rate (CIR) in bits per second.

Usage example:
```bash
./forward -- -m 2
```
Note: This commadn required `-s` option.

#### 2.6.1.10 Data rate limits
Use `-l` command to configure meter profile limits for ingress and egress ports.

Syntax:
```bash
-l
```

Usage example:
```bash
./forward -- -l
```

Note: This commadn required `-m` option.

#### 2.6.1.11 Verbose logging
Use `-d` command to display verbose logs of packets reading and learning.

Syntax:
```bash
-d
```

Usage example:
```bash
./forward  -- -d
```

### 2.6.2 Application also contains internal CLI arguments for operation
```bash
Press u or send SIGNAL 10 to unlearn flows
Press l or send SIGNAL 12 to learn flows
Press 1 or send SIGNAL 40 to delete default filters
Press 2 or send SIGNAL 41 to setup default filters
Press q or send SIGNAL 15 to quit
```

#### 2.6.2.1 Unlearn flows
Use `u` command to unlern all flows from the HW.

Usage example:
```bash
u
```

#### 2.6.2.2 Learn flows
Use `l` command to start flow lerning. All futher packets will be learned

Usage example:
```bash
l
```

#### 2.6.2.3 Delete default filters
Use `1` command to delete default filters.

Usage example:
```bash
1
```

Default filters includes:
* IPV4 filter
* IPV6 filter
* Queues filter

#### 2.6.2.4 Setup default filters
Use `2` command to setup default filters.

Usage example:
```bash
2
```

#### 2.6.2.5 Quit
Use `q` command to quit from the applocation.

Usage example:
```bash
q
```

Note: All flows will be destroyed automatically

# 3. Traffic generator How-to instructions (Node 1)

This section includes guidelines on how to prepare
Node 1 to work with traffic generator tools.

There are 2 supported options on how to send traffic to demo application:
- re-play prepared PCAP file;
- use TREX traffic generator.

Note: Preparation and Usage of traffic generator tools require
'Napatech Link-Capture™' Software and the 'Napatech libpcap'
to be installed.

Detailed instructions on how to install necessary software:
https://docs.napatech.com/r/Link-CaptureTM-Software-Getting-Started-Guide/Installing-Napatech-Link-CaptureTM-Software

Detailed instructions on how to run traffic generator tools: [./traffic_gen_tools_conf/README.md](./traffic_gen_tools_conf/README.md).
