/*
 * BSD LICENSE
 *
 * Copyright(c) 2010-2015 Intel Corporation. All rights reserved.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of Intel Corporation nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INLINE_UPF_DEMO_FILTER_H
#define INLINE_UPF_DEMO_FILTER_H

#include <rte_flow.h>

#define PORT_UL 0
#define PORT_DL 1
#define DEFAULT_DSCP 0x00
#define DEFAULT_QFI  0x01

#define MAX_DECAP_SIZE 128
#define MAX_ENCAP_SIZE 128

extern size_t decap_ipv4_size;
extern size_t encap_ipv4_size;
extern uint8_t decap_ipv4_buf[MAX_DECAP_SIZE];
extern uint8_t encap_ipv4_buf[MAX_ENCAP_SIZE];

extern size_t decap_ipv6_size;
extern size_t encap_ipv6_size;
extern uint8_t decap_ipv6_buf[MAX_DECAP_SIZE];
extern uint8_t encap_ipv6_buf[MAX_ENCAP_SIZE];

extern const uint32_t N6_IF_IPV4_NET;

extern const uint8_t N6_IF_IPV6[16];

extern const uint16_t NAT_ports[2];

enum flow_match_groups {
  KEY_MATCH_GROUP = 0,
  FLOW_MATCH_IPV4_GROUP_UPLINK,
  FLOW_MATCH_IPV6_GROUP_UPLINK,
  FLOW_MATCH_IPV4_GROUP_DOWNLINK,
  FLOW_MATCH_IPV6_GROUP_DOWNLINK,
};

int init_GTP_tun_template(const uint32_t n3_ipv4,
                          const uint32_t n3_ipv4_peer,
                          const uint8_t n3_if_ipv6[16],
                          const uint8_t n3_if_ipv6_peer[16]);

struct rte_flow* setup_main_uplink_IPv4_filter(uint8_t port_id,
  uint32_t match_ipv4_src,
  uint32_t match_ipv4_src_mask,
  uint32_t match_ipv4_dst,
  uint32_t match_ipv4_dst_mask,
  uint32_t flow_match_group);

struct rte_flow* setup_main_downlink_IPv4_filter(uint8_t port_id,
  uint32_t match_ipv4_src,
  uint32_t match_ipv4_src_mask,
  uint32_t match_ipv4_dst,
  uint32_t match_ipv4_dst_mask,
  uint32_t flow_match_group);

struct rte_flow* setup_tunnel_IPv4_downlink_flow(uint8_t port_id,
  uint32_t match_ipv4_src,
  uint32_t match_ipv4_dst,
  uint16_t match_udp_src,
  uint16_t match_udp_dst,
  uint32_t UE_ipv4,
  uint16_t NAT_udp_src,
  uint32_t encap_gtp_teid,
  uint8_t qfi,
  bool do_NAT,
  uint8_t action_out_port_id,
  uint32_t flow_match_group,
  uint32_t *meter_id_arr, int size);

struct rte_flow* setup_tunnel_IPv4_uplink_flow(uint8_t port_id,
  uint32_t match_ipv4_src,
  uint32_t match_ipv4_dst,
  uint16_t match_udp_src,
  uint16_t match_udp_dst,
  uint32_t match_gtp_teid,
  uint8_t dscp_tag,
  bool do_NAT,
  uint32_t N6_ipv4_addr,
  uint16_t NAT_udp_src,
  uint8_t action_out_port_id,
  uint32_t flow_match_group,
  uint32_t *meter_id_arr, int size);

struct rte_flow* setup_main_uplink_IPv6_filter(uint8_t port_id,
  const uint8_t match_ipv6_src[16],
  const uint8_t match_ipv6_src_mask[16],
  const uint8_t match_ipv6_dst[16],
  const uint8_t match_ipv6_dst_mask[16],
  uint32_t flow_match_group);

struct rte_flow* setup_main_downlink_IPv6_filter(uint8_t port_id,
  const uint8_t match_ipv6_src[16],
  const uint8_t match_ipv6_src_mask[16],
  const uint8_t match_ipv6_dst[16],
  const uint8_t match_ipv6_dst_mask[16],
  uint32_t flow_match_group);

struct rte_flow* setup_tunnel_IPv6_uplink_flow(uint8_t port_id,
  const uint8_t match_ipv6_src[16],
  const uint8_t match_ipv6_dst[16],
  uint16_t match_udp_src,
  uint16_t match_udp_dst,
  uint32_t match_gtp_teid,
  uint8_t dscp_tag,
  bool do_NAT,
  uint16_t NAT_udp_src,
  uint8_t action_out_port_id,
  uint32_t flow_match_group,
  uint32_t *meter_id_arr, int size);

struct rte_flow* setup_tunnel_IPv6_downlink_flow(uint8_t port_id,
  const uint8_t match_ipv6_src[16],
  const uint8_t match_ipv6_dst[16],
  uint16_t match_udp_src,
  uint16_t match_udp_dst,
  const uint8_t UE_ipv6[16],
  uint16_t NAT_udp_src,
  uint32_t encap_gtp_teid,
  uint8_t qfi,
  bool do_NAT,
  uint8_t action_out_port_id,
  uint32_t flow_match_group,
  uint32_t *meter_id_arr, int size);

struct rte_flow* setup_rss_uplink_flow(uint8_t port,
  uint16_t* queues,
  uint32_t queue_num,
  uint32_t flow_match_group);

#endif  /* INLINE_UPF_DEMO_FILTER_H */
