/*
 * BSD LICENSE
 *
 * Copyright(c) 2010-2015 Intel Corporation. All rights reserved.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of Intel Corporation nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "filter.h"

#include <rte_ethdev.h>
#include <rte_flow.h>

/* Definitions of external variables */
uint8_t decap_ipv4_buf[MAX_DECAP_SIZE];
uint8_t encap_ipv4_buf[MAX_ENCAP_SIZE];

uint8_t decap_ipv6_buf[MAX_DECAP_SIZE];
uint8_t encap_ipv6_buf[MAX_ENCAP_SIZE];


int init_GTP_tun_template(const uint32_t n3_ipv4,
                          const uint32_t n3_ipv4_peer,
                          const uint8_t n3_if_ipv6[16],
                          const uint8_t n3_if_ipv6_peer[16])
{
  /* TODO: Add functionality for vlan */

  /* Encap ipv4 */
  {
    struct rte_ether_hdr eth = {
      .dst_addr.addr_bytes = "\x16\x16\x16\x16\x16\x16",
      .src_addr.addr_bytes = "\x1A\x1A\x1A\x1A\x1A\x1A",
      .ether_type = RTE_BE16(RTE_ETHER_TYPE_IPV4) };

    struct rte_ipv4_hdr ipv4 = {
      .version_ihl = 0x45,
      .src_addr = n3_ipv4,
      .dst_addr = n3_ipv4_peer,
      .type_of_service = 10,
      .time_to_live = 0x40,
      .next_proto_id = IPPROTO_UDP };

    struct rte_udp_hdr udp = {
      .src_port = RTE_BE16(2152),
      .dst_port = RTE_BE16(2152) };

    struct rte_gtp_hdr gtp = {
      .ver = 1,
      .pt = 1,
      .e = 1,
      .msg_type = 255,
      .teid = RTE_BE32(0x00000000) };

    struct rte_gtp_hdr_ext_word gtp_ext = {
      .next_ext = 0x85 };

    struct gtp_pdu_session_container {
      uint8_t ext_hdr_len;
#if RTE_BYTE_ORDER == RTE_BIG_ENDIAN
      uint8_t type:4;
      uint8_t spare:4;
      uint8_t ppp:1;
      uint8_t rqi:1;
      uint8_t qfi:6;
#else
      uint8_t spare:4;
      uint8_t type:4;
      uint8_t qfi:6;
      uint8_t rqi:1;
      uint8_t ppp:1;
#endif
      uint8_t next_ext;
    } gtp_psc = {
      .ext_hdr_len = 1,
      .type = 1,
      .spare = 0,
      .ppp = 0,
      .rqi = 1,
      .qfi = 6,
      .next_ext = 0 };

    memcpy(encap_ipv4_buf + encap_ipv4_size, &eth, sizeof(eth));
    encap_ipv4_size += sizeof(eth);

    memcpy(encap_ipv4_buf + encap_ipv4_size, &ipv4, sizeof(ipv4));
    encap_ipv4_size += sizeof(ipv4);

    memcpy(encap_ipv4_buf + encap_ipv4_size, &udp, sizeof(udp));
    encap_ipv4_size += sizeof(udp);

    memcpy(encap_ipv4_buf + encap_ipv4_size, &gtp, sizeof(gtp));
    encap_ipv4_size += sizeof(gtp);

    memcpy(encap_ipv4_buf + encap_ipv4_size, &gtp_ext, sizeof(gtp_ext));
    encap_ipv4_size += sizeof(gtp_ext);

    memcpy(encap_ipv4_buf + encap_ipv4_size, &gtp_psc, sizeof(gtp_psc));
    encap_ipv4_size += sizeof(gtp_psc);
  }

  /* Decap ipv4 */
  {
    struct rte_ether_hdr eth = {
      .dst_addr.addr_bytes = "\x17\x17\x17\x17\x17\x17",
      .src_addr.addr_bytes = "\x18\x18\x18\x18\x18\x18",
      .ether_type = RTE_BE16(RTE_ETHER_TYPE_IPV4) };

    memcpy(decap_ipv4_buf, &eth, sizeof(eth));
    decap_ipv4_size += sizeof(eth);
  }

  /* Encap ipv6 */
  {
    struct rte_ether_hdr eth = {
      .dst_addr.addr_bytes = "\x16\x16\x16\x16\x16\x16",
      .src_addr.addr_bytes = "\x1A\x1A\x1A\x1A\x1A\x1A",
      .ether_type = RTE_BE16(RTE_ETHER_TYPE_IPV6) };

    struct rte_ipv6_hdr ipv6 =
    {
      .src_addr = {0},
      .dst_addr = {0},
      .hop_limits = 0x40,
      .proto = IPPROTO_UDP,
      .vtc_flow = RTE_BE32(0x60000000)
    };

    memcpy(ipv6.src_addr, n3_if_ipv6, 16);
    memcpy(ipv6.dst_addr, n3_if_ipv6_peer, 16);

    struct rte_udp_hdr udp = {
      .dst_port = RTE_BE16(2152) };

    struct rte_gtp_hdr gtp = {
      .ver = 1,
      .pt = 1,
      .e = 1,
      .msg_type = 255,
      .teid = RTE_BE32(0x00000000) };

    struct rte_gtp_hdr_ext_word gtp_ext = {
      .next_ext = 0x85 };

    struct gtp_pdu_session_container {
      uint8_t ext_hdr_len;
#if RTE_BYTE_ORDER == RTE_BIG_ENDIAN
      uint8_t type:4;
      uint8_t spare:4;
      uint8_t ppp:1;
      uint8_t rqi:1;
      uint8_t qfi:6;
#else
      uint8_t spare:4;
      uint8_t type:4;
      uint8_t qfi:6;
      uint8_t rqi:1;
      uint8_t ppp:1;
#endif
      uint8_t next_ext;
    } gtp_psc = {
      .ext_hdr_len = 1,
      .type = 1,
      .spare = 0,
      .ppp = 0,
      .rqi = 1,
      .qfi = 6,
      .next_ext = 0 };

    memcpy(encap_ipv6_buf + encap_ipv6_size, &eth, sizeof(eth));
    encap_ipv6_size += sizeof(eth);

    memcpy(encap_ipv6_buf + encap_ipv6_size, &ipv6, sizeof(ipv6));
    encap_ipv6_size += sizeof(ipv6);

    memcpy(encap_ipv6_buf + encap_ipv6_size, &udp, sizeof(udp));
    encap_ipv6_size += sizeof(udp);

    memcpy(encap_ipv6_buf + encap_ipv6_size, &gtp, sizeof(gtp));
    encap_ipv6_size += sizeof(gtp);

    memcpy(encap_ipv6_buf + encap_ipv6_size, &gtp_ext, sizeof(gtp_ext));
    encap_ipv6_size += sizeof(gtp_ext);

    memcpy(encap_ipv6_buf + encap_ipv6_size, &gtp_psc, sizeof(gtp_psc));
    encap_ipv6_size += sizeof(gtp_psc);
  }

  /* Decap ipv6 */
  {
    struct rte_ether_hdr eth = {
      .dst_addr.addr_bytes = "\x17\x17\x17\x17\x17\x17",
      .src_addr.addr_bytes = "\x18\x18\x18\x18\x18\x18",
      .ether_type = RTE_BE16(RTE_ETHER_TYPE_IPV6) };

    memcpy(decap_ipv6_buf, &eth, sizeof(eth));
    decap_ipv6_size += sizeof(eth);
  }

  return 0;
}

struct rte_flow* setup_rss_uplink_flow(uint8_t port,
  uint16_t* queues,
  uint32_t queue_num,
  uint32_t flow_match_group)
{
  struct rte_flow_attr attr;
  struct rte_flow_error error;

  struct rte_flow_item patterns[16];
  struct rte_flow_action actions[16];

  struct rte_flow *flow_filter = NULL;
  int action_it = 0;
  int pattern_it = 0;

  memset(&attr, 0x0, sizeof(attr));
  memset(&error, 0x0, sizeof(error));

  attr.group = flow_match_group;

  /* Pattern: End */
  patterns[pattern_it].type = RTE_FLOW_ITEM_TYPE_END;
  patterns[pattern_it].spec = NULL;
  patterns[pattern_it].mask = NULL;
  patterns[pattern_it].last = NULL;
  pattern_it += 1;

  /* Action: RSS */
  struct rte_flow_action_rss rss;

  rss.queue_num = queue_num;
  rss.queue = queues;
  rss.func = RTE_ETH_HASH_FUNCTION_DEFAULT;
  rss.types = RTE_ETH_RSS_UDP;
  rss.level = 2; /* Inner packet */
  rss.key = NULL;
  rss.key_len = 0;

  actions[action_it].type = RTE_FLOW_ACTION_TYPE_RSS;
  actions[action_it].conf = &rss;
  action_it += 1;

  /* Action: End */
  actions[action_it].type = RTE_FLOW_ACTION_TYPE_END;
  actions[action_it].conf = NULL;
  action_it += 1;

  flow_filter = rte_flow_create(port, &attr, patterns, actions, &error);

  if (!flow_filter) {
    printf("Error: Cannot create main RSS UpLink filter: %s\n", error.message);
  }

  return flow_filter;
}
