#!/usr/bin/env bash

NT_PF_DEV_ARRAY=($NT_PF_DEV)
NT_VF_NUM_ARRAY=($NT_VF_NUM)
NT_VF_MAC_ARRAY=($NT_VF_MAC)
NT_VF_SOCKET_ARRAY=($NT_VF_SOCKET)

DPDK_ARGS=${DpdkArgs}
APP_ARGS=${AppArgs}
APP_BIN="inline_upf_demo"

echo -n "POD Configuration from Napatech Device Plugin for resource 'napatech.com/nt_"

function catch_sigterm() {
        echo
        echo "Caught SIGTERM signal, ${APP_BIN} will be terminated."
        kill -TERM ${APP_PID} 2>/dev/null
}

if [ "$NT_PF_DEV" != "" ] ; then
        #
        # app with NTNIC PMD will be executed
        #
        VF_TOKEN="$(uuidgen)"
        echo "pf: 1'"
        echo "  NT_PF_DEV =       $NT_PF_DEV"
        echo "  VF_TOKEN =        $VF_TOKEN"
        APP_CPU="-c 0xfffe"
        DPDK_ARGS="${DPDK_ARGS} --vfio-vf-token=${VF_TOKEN} -a ${NT_PF_DEV_ARRAY[0]}"
else
        #
        # app with Virtio PMD will be executed
        #
        echo "vf: ${#NT_VF_NUM_ARRAY[@]}'"
        # sanity check - two ports are required
        if [ ${#NT_VF_NUM_ARRAY[@]} -ne 2 ] ; then
                echo "ERROR: app requires 2 VFs, but '${#NT_VF_NUM_ARRAY[@]}' VFs were allocated by Napatech Device Plugin"
                exit 1
        fi
        APP_CPU="-c 0xfff0"
        DPDK_ARGS="${DPDK_ARGS} --no-pci"
        for i in ${!NT_VF_MAC_ARRAY[@]}; do
                DPDK_ARGS="${DPDK_ARGS} --vdev=virtio_user${i},path=${NT_VF_SOCKET_ARRAY[$i]},server=1,mac=${NT_VF_MAC_ARRAY[$i]},packed_vq=1,queues=${NT_VF_QUEUES}"
                # remove old socket
                rm -f ${NT_VF_SOCKET_ARRAY[$i]}
        done
fi

# Configuration common for both PF and VF resources
echo "  NT_VF_NUM =       $NT_VF_NUM"
echo "  NT_VF_QUEUES =    $NT_VF_QUEUES"
echo "  NT_VF_DEV =       $NT_VF_DEV"
echo "  NT_VF_MAC =       $NT_VF_MAC"
echo "  NT_VF_VLAN =      $NT_VF_VLAN"
echo "  NT_VF_SOCKET =    $NT_VF_SOCKET"
echo "  NT_DPDK_PREFIX =  $NT_DPDK_PREFIX"
echo "Configuration from configmap napatech-inline-upf-demo-config:"
echo "  DpdkArgs  =       ${DpdkArgs}"
echo "  AppArgs   =       ${AppArgs}"

# Command line
cmd="${APP_BIN} ${APP_CPU} -n 4 --file-prefix=${NT_DPDK_PREFIX} ${DPDK_ARGS} -- ${APP_ARGS}"

trap catch_sigterm SIGTERM

# Call the command
echo "Executing: ${cmd}"
${cmd} &
APP_PID=$!
echo "Wait for ${APP_BIN} with PID ${APP_PID}"
wait ${APP_PID}
echo "${APP_BIN} has been terminated"
