#!/usr/bin/env bash

function catch_sigterm() {
        echo
        echo "Caught SIGTERM signal, ${APP_BIN} will be terminated."
        kill -TERM ${APP_PID} 2>/dev/null
}

trap catch_sigterm SIGTERM

# Exit if Main App is not enabled by configuration
if [ "${MainApp}" != "enabled" ] ; then
    echo "NT Main App is disabled by configuration."
    APP_BIN="container"         # used at catch_sigterm() log message
    sleep infinity &
    wait
    exit 0
fi

# Configuration
NT_VENDOR_ID="0x18f4"
APP_BIN="ntmain"
APP_CPU="-c 0xfffe"             # if needed, specify cpus to be used, e.g. "-c 0xfffe" or "-l 4,5,10-15"
VF_VLAN_START="${vfVlanStart}"
VF_NUM="${vfCount}"
NT_NIC="${NicPci}"
VF_TOKEN="$(uuidgen)"
if [ "$NT_NIC" == "autodetect" ] ; then
    NT_NIC_AUTO=$(grep -l ${NT_VENDOR_ID} /sys/bus/pci/devices/*.0/vendor | head -n1 | sed -E 's|/sys/bus/pci/devices/([0-9:.]+)/vendor|\1|')
    NT_NIC=${NT_NIC_AUTO}
fi
NT_VF_DEV_ARRAY=($(for VF in $(readlink /sys/bus/pci/devices/${NT_NIC}/virtfn*) ; do echo $(basename $VF) ; done | sort))
NT_DEVICE_ID=$(cat /sys/bus/pci/devices/${NT_NIC}/device)
case "${NT_DEVICE_ID}" in
    "0x0215")
        echo "NT400D was detected."
        VF_FIRST=1
        ;;
    "0x01c5")
        echo "NT200A was detected."
        VF_FIRST=4
        ;;
    *)
        echo "Unknown NT SmartNIC type ${NT_DEVICE_ID} was detected."
        VF_FIRST=4
        ;;
esac

NT_DPDK_PREFIX="nt-main-${NT_NIC}"
DPDK_ARGS=""
COMMA=""
for i in ${!NT_VF_DEV_ARRAY[@]}; do
    DPDK_ARGS="${DPDK_ARGS} -a ${NT_VF_DEV_ARRAY[$i]},vlan=$(($VF_VLAN_START+$i)),sep=1"
    PORTQUEUES="${PORTQUEUES}${COMMA}$((${VF_FIRST}+$i)):${vfNumberOfQueues}"
    COMMA=","
done
DPDK_ARGS="--vfio-vf-token=${VF_TOKEN} -a ${NT_NIC},exception_path=1,portqueues=[${PORTQUEUES}] ${DPDK_ARGS}"

# Configuration common for both PF and VF resources
echo "Execution of NT main APP:"
echo "  NT_NIC =          $NT_NIC"
echo "  NT_VF_DEV =       ${NT_VF_DEV_ARRAY[@]}"
echo "  NT_DPDK_PREFIX =  $NT_DPDK_PREFIX"
echo "  NT_VF_NUM =       ${vfCount}"
echo "  NT_VF_QUEUES =    ${vfNumberOfQueues}"
echo "  VF_VLAN_START =   $VF_VLAN_START"
echo "  VF_TOKEN =        $VF_TOKEN"

# Command line
cmd="${APP_BIN} ${APP_CPU} -n 4 --file-prefix=${NT_DPDK_PREFIX} ${DPDK_ARGS} -- --queues ${vfNumberOfQueues}"

# Call the command
echo "Executing: ${cmd}"
${cmd} &
APP_PID=$!
echo "Wait for ${APP_BIN} with PID ${APP_PID}"
wait ${APP_PID}
