/*
 * SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef _STATISTIC_TOOL_H_
#define _STATISTIC_TOOL_H_

#include "ntconn_client.h"
#include "ntconnect_api_statistic.h"
#include "ntconnect_api_adapter.h"

#define QUEUE_HEADER      "  Queue  Owner   Dest           Packets              Octets      VQ Drop pkts      VQ Drop octets     "
#define COLOR_STAT_HEADER "  Color              Packets               Bytes    tcp flags                                         "

#define MAX_QUEUE_NUM   128
#define QUEUE_FORMAT "  %4d   %12s  #%015lu  #%017lu  #%015lu  #%017lu"
#define COLOR_STAT_FORMAT "  %5d     #%015lu  #%017lu        %02x"

#define PARAM_SIZE 96

static const char *const usage[] = {
  "The Statistic tool commands used to get specific statistic information\n"
  "\n"
  "Syntax:\n"
  "---------------------\n"
  "\n"
  "statistic [-a adapter] [-p port] [-g param] [-h]\n"
  "\n",
  NULL,
};

static const char *usageExamples =
  "\nImplemented commands:\n"
  "* statistic -p 0 -g txrx_stats\n"
  "\n"
  "* statistic -g que_stats\n"
  "\n"
  "* statistic -g color_stats\n"
  "\n"
  "* statistic -g flm_stats\n"
  "\n";

struct stat_data_s {
  struct ntc_nic_pci_ids_s *nics;
  struct ntc_interfaces_s *ifs;

  int adap_sel;
  int stat_layout_version;
  int stat_flm_layout_version;
  uint64_t stat_version;
  struct ntc_stat_get_data_s *clr;
  struct ntc_stat_get_data_s *que;
  struct ntc_stat_get_data_s *rxc;
  struct ntc_stat_get_data_s *txc;
  struct ntc_stat_get_data_s *flm;
};

#endif /* _STATISTIC_TOOL_H_ */
